/**
 * Last Modified by: $Author: rjeffers $
<<<<<<< CSIFactory.java
 * On: $Date: 2005/12/13 04:33:51 $
=======
 * On: $Date: 2005/12/13 04:33:51 $
>>>>>>> 1.23
 */

package kaos.core.csi;

import kaos.core.csi.extension.classification.InstanceClassifierManager;
import kaos.core.csi.extension.classification.InstanceClassifierManagerImpl;
import kaos.core.csi.extension.history.logging.HistoryMonitor;
import kaos.core.csi.observe.ObserverManager;
import kaos.core.csi.observe.ObserverManagerImpl;
import kaos.core.csi.query.QueryRegistration;
import kaos.core.csi.query.QueryRegistrationImpl;
import kaos.core.csi.query.QueryState;
import kaos.core.csi.policy.PolicyAdvice;
import kaos.core.csi.query.QueryStateImpl;
import kaos.core.csi.policy.PolicyCheckingImpl;
import kaos.core.csi.policy.PolicyChecking;
import kaos.core.csi.policy.PolicyManagementImpl;
import kaos.core.csi.policy.PolicyManagement;
import kaos.core.csi.policy.enforcement.EnforcerManager;
import kaos.core.csi.policy.enforcement.EnforcerManagerImpl;
import kaos.core.csi.registration.Registration;
import kaos.core.csi.registration.RegistrationImpl;
import kaos.core.csi.request.RequestManager;
import kaos.core.csi.request.RequestManagerImpl;
import kaos.core.csi.transport.TransportSupport;
import kaos.core.csi.transport.TransportSupportImpl;
import kaos.core.csi.transport.PolicyEnforcedTransportSupportImpl;
import kaos.core.csi.util.xmlSupport.XMLToCSILink;
import kaos.policy.guard.GuardRetriever;
import kaos.policy.guard.KAoSGuard;
import kaos.spatial.SpatialReasoner;
import kaos.spatial.LocalSpatialReasoner;

/**
 * This class provides access to specific servicies offered by the KAoS Common Servicies Interface
 * 
 * $Revision: 1.27 $
 */
public class CSIFactory
{    
    public static final String TCP_TRANSPORT = "tcp";
    public static final String CORBA_TRANSPORT = "corba";
    public static final String COABSGRID_TRANSPORT = "grid";
    
    private static String currentTransportName = TCP_TRANSPORT;
    
    /**
     * @return Returns the currentTransportName.
     */
    public static String getCurrentTransport() 
    {
        return currentTransportName;
    }

    /**
     * @param currentTransportName The currentTransportName to set.
     */
    public static void setCurrentTransport (String currentTransportName) 
    {
        CSIFactory.currentTransportName = currentTransportName;
    }

    /**
     * Retrieve a reference to the ObserverManager.
     *      
     * @return a reference to the ObserverManager Service
     */
	public static ObserverManager getObserverManager() throws Exception
    {   
        return ObserverManagerImpl.getInstance(currentTransportName);
    }

    /**
     * Retrieve a reference to the ObserverManager.
     *                                   
	 * @param       nickName        is a String containing  the nick name of the Command that may be used by certain
	 *                              transport implementations, or null if not required by the transport.
     *
     * @return a reference to the ObserverManager
     */
	public static ObserverManager getObserverManager(String nickName) throws Exception
    {   
        return ObserverManagerImpl.getInstance(currentTransportName, nickName);
    }
    
	/**
     * Retrieve a reference to the Registration.
     * 
     * @return a reference to the Registration Service
     */
    public static Registration getRegistration() throws Exception
    {        
        return RegistrationImpl.getInstance(currentTransportName);
    }
    
    /**
     * Retrieve a reference to the PolicyManagement.
     * 
     * @return a reference to the PolicyManagement
     */
    public static PolicyManagement getPolicyManagement() throws Exception
    {        
        return PolicyManagementImpl.getInstance(currentTransportName);
    }
    
    /**
     * Retrieve a reference to the PolicyChecking.
     * 
     * @return a reference to the PolicyChecking
     */
    public static PolicyChecking getPolicyChecking()throws Exception
    {        
        return PolicyCheckingImpl.getInstance(currentTransportName);
    }
    
    /**
     * Retrieve a reference to the PolicyAdvice.
     * 
     * @return a reference to the PolicyAdvice.
     */
    public static PolicyAdvice getPolicyAdvice()throws Exception
    {        
        return PolicyCheckingImpl.getPolAdviceInstance(currentTransportName);
    }
    
    /**
     * Retrieve a reference to the PolicyEnforcement.
     * 
     * @return a reference to the PolicyEnforcement
     */
    public static EnforcerManager getPolicyEnforcement() throws Exception
    {        
        return EnforcerManagerImpl.getInstance();
    }
    
    /**
     * Retrieve a reference to the ClassifierManagement
     * 
     * @return a reference to the PolicyEnforcement
     */
    public static InstanceClassifierManager getClassifierManagement() throws Exception
    {        
        return (InstanceClassifierManager)InstanceClassifierManagerImpl.getInstance();
    }
    
    /*
     * Retrieve a reference to the QueryRegistration.
     * 
     * @return a reference to the QueryRegistration.
     */
    public static QueryRegistration getQueryRegistration() throws Exception
    {        
        return QueryRegistrationImpl.getInstance(currentTransportName);
    }
    
    /**
     * Retrieve a reference to the QueryState.
     * 
     * @return a reference to the QueryState.
     */
    public static QueryState getQueryState() throws Exception
    {        
        return QueryStateImpl.getInstance(currentTransportName);
    }
    
    /**
     * Retrieve a reference to the Registration.
     * 
     * @return a reference to the Registration
     */
    public static TransportSupport getTransportSupport()
    {
       if ( Boolean.getBoolean("kaos.comm.policy.enforce") )
           return PolicyEnforcedTransportSupportImpl.getInstance();
       else 
           return TransportSupportImpl.getInstance();
    }
    
    /**
     * Retrieve a reference to the RequestManager.
     * 
     * @return a reference to the RequestManager
     */
    public static RequestManager getRequestManager() throws Exception
    {
        return RequestManagerImpl.getInstance(currentTransportName);
    }
    
    /**
     * Retrieve a reference to the SpatialReasoner.
     * 
     * @return a reference to the SpatialReasoner. 
     */
    public static SpatialReasoner getSpatialReasoner() throws Exception
    {
        if ( (System.getProperty(SpatialReasoner.GLOBAL_SPATIAL_REASONER)) != null )
            return SpatialReasoner.getInstance(currentTransportName);
        else
            return LocalSpatialReasoner.getLSRInstance(currentTransportName);
    }
    
    /**
     * Retrieve a reference to the XMLToCSILink.
     * 
     * @return a reference to the XMLToCSILink
     */
    public static XMLToCSILink getXMLToCSILink(String dsTransport) throws Exception
    {
        return XMLToCSILink.getInstance(dsTransport);
    } 
    
    /**
     * Retrieve a reference to the RequestManager.
     * 
     * @return a reference to the RequestManager
     */
    public static HistoryMonitor getHistoryMonitor() throws Exception
    {
    	KAoSGuard guard = (KAoSGuard)GuardRetriever.getGuard();

		if (guard == null)
		{
			System.out.println("\n Unable to History Monito!r");
			return null;
		}
		
        return guard.getGuardLogManager();
    }
    
}
