/**
 * Last Modified by: $Author: auszok $
 * On: $Date: 2006/01/23 22:56:00 $
 */
 
package kaos.core.csi;

import java.util.Set;
import java.net.URISyntaxException;

import kaos.ontology.util.SerializableOntModelImpl;

/**
 *   This interface containes methods allowing to create and investigate a description of an Ontology instance.
 *   
 *   $Revision: 1.2 $
 */
public interface OntInstanceDescription extends Cloneable
{   
    /** 
     * Add the ontology type of this instance.
     *
     * @param name                String containing the uri of the ontology type (class). 
     * 
     * @throws URISyntaxException when the name is not a correct URI
     */
    public void addType(String name) throws URISyntaxException;
    
    /** 
     * Add property to the instance
     *
     * @param newProperty  OntPropertyDescription with the property description. 
     */
    public void addProperty(OntPropertyDescription newProperty);
    
    /** 
     * Get name of the instance
     *
     * @return             String representing the name in URI format. 
     */
    public String getName();
    
    /** 
     * Get types of the instance
     *
     * @return             Set containing the strings with URIs of the types. 
     */
    public Set getTypes();
    
    /** 
     * Get properties of the instance
     *
     * @return             Set containing the OntPropertyDescriptions. 
     */
    public Set getProperties();
    
    /** 
     * Get property of the instance
     *
     * @param propertyName name of the property. 
     *
     * @return             OntPropertyDescription. 
     */
    public OntPropertyDescription getProperty(String propertyName);
    
    /** 
     * Remove the ontology type of this instance.
     *
     * @param name      String containing the uri of the ontology type (class). 
     */
    public void removeType(String name);
    
    /** 
     * Get property from the instance description
     *
     * @param propertyName  property name. 
     */
    public void removeProperty(String propertyName);
    
    /**
     * Clone method
     * 
     * @return copy of the object
     */
    public Object clone();
    
    /**
     * Get the Jena-based Ontology Model object for this instance.
     * 
     * @return Jena-based Ontology Model
     */
    public SerializableOntModelImpl getInstanceOntModel();
}