/**
 * Last Modified by: $Author: rjeffers $
 * On: $Date: 2006/01/16 18:43:05 $
 */
package kaos.core.csi;

import java.util.Iterator;
import java.util.Set;
import java.util.HashSet;
import java.util.Map;
import java.util.Hashtable;
import java.io.Serializable;
import java.net.URI;
import java.net.URISyntaxException;

import kaos.core.util.Logger;
import kaos.ontology.vocabulary.ActionConcepts;
import kaos.ontology.vocabulary.ActionStatusConcepts;
/**
 * 
 * This is an implementaion of the interface OntPropertyDescription
 * 
 * $Revision: 1.6 $
 */
public class OntPropertyDescriptionImpl implements OntPropertyDescription, Cloneable , Serializable
{
	public static final String	OBLIGATION_VIOLATION_PROPERTY_NAME = ActionStatusConcepts.hasReasonForUnfulfillment();
    /**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = 1944343282020579753L;
    private String myName;
    private Set myValues;
    private Map myValueDesc;
    
    public static OntPropertyDescription createObligationViolationProperty(String reason)
    {
    	OntPropertyDescription violatedObligationProp = null;
    	try {
    		violatedObligationProp = new OntPropertyDescriptionImpl(OBLIGATION_VIOLATION_PROPERTY_NAME);
        } catch (URISyntaxException e) {
        	// this should never happen, since the ActionStatusConcepts.hasReasonForUnfulfillment() is valid....
            // TODO Auto-generated catch block
            e.printStackTrace();          
        }
    	
        violatedObligationProp.addValue(reason);
        return violatedObligationProp;
    	
    }
    /**
     * Constructor with instance name
     * 
     * @param name - instance name
     */
    public OntPropertyDescriptionImpl(String name) throws URISyntaxException
    {
        super();
        URI testURI = new URI(name);
        
        myName = name;
        myValues = new HashSet();
        myValueDesc = new Hashtable();
    }
    
    /* 
     * @see kaos.core.csi.OntPropertyDescription#addValue(java.lang.String)
     */
    public void addValue(String value) 
    {
        myValues.add(value);
    }

    /* 
     * @see kaos.core.csi.OntPropertyDescription#setValueDescForValue(java.lang.String, java.lang.Object)
     */
    public void setValueDescForValue(String value, Object desc) 
    {
        if (!myValues.contains(value))
            addValue(value);
        
        myValueDesc.put(value, desc);
    }

    /* 
     * @see kaos.core.csi.OntPropertyDescription#getName()
     */
    public String getName() 
    {
        return myName;
    }

    /* 
     * @see kaos.core.csi.OntPropertyDescription#getValues()
     */
    public Set getValues() 
    {
        return myValues;
    }

    /* 
     * @see kaos.core.csi.OntPropertyDescription#getValueDescForValue(java.lang.String)
     */
    public Object getValueDescForValue(String value) 
    {
        return myValueDesc.get(value);
    }
    
    /* 
     * @see kaos.core.csi.OntPropertyDescription#removeValue(java.lang.String)
     */
    public void removeValue(String value) 
    {
        if (!myValues.contains(value))
            return;
            
        myValueDesc.remove(value);
        myValues.remove(value);
    }

    /* 
     * @see kaos.core.csi.OntPropertyDescription#removeValueDescForValue(java.lang.String)
     */
    public void removeValueDescForValue(String value) 
    {
        myValues.remove(value);
    }
    
    public  Object clone()
    {       
        OntPropertyDescriptionImpl myClone = null;               
        
        try 
        {
            myClone = (OntPropertyDescriptionImpl) super.clone();
        }
        catch (CloneNotSupportedException e) 
        {
            // this should never happen
            throw new InternalError(e.toString());
        }
        
        for(Iterator it = myValueDesc.keySet().iterator(); it.hasNext(); )
        {
            String currentPropertyWithDescription = (String)it.next();
            
            Object currentPropertyDescription = myValueDesc.get(currentPropertyWithDescription);
            
            //TODO: currently supports only cloning of value description of class OntInstanceDescriptionImpl
            if(currentPropertyDescription instanceof OntInstanceDescriptionImpl)
                myClone.setValueDescForValue(currentPropertyWithDescription, ((OntInstanceDescriptionImpl)currentPropertyDescription).clone());
        }
        
        return myClone; 
    }
    
    public boolean equals (Object obj)
    {
        if (obj instanceof OntPropertyDescriptionImpl) {
            OntPropertyDescriptionImpl other = (OntPropertyDescriptionImpl) obj;
            boolean test1 = other.myName.equals(myName);
            boolean test2 = other.myValues.equals(myValues);
            boolean test3 = other.myValueDesc.equals(myValueDesc);
            return (other.myName.equals(myName) &&
                    other.myValues.equals(myValues) &&
                    other.myValueDesc.equals(myValueDesc));
        }
        return false;
    }
    
    public int hashCode()
    {
        return myName.hashCode() & myValues.hashCode() & myValueDesc.hashCode();
    }
    
    /* 
     * Returns a String representation of this instance.
     */
    public String toString()
    {
       StringBuffer buffer = new StringBuffer();
	   buffer.append("\n\tOntology Property Description:");
	   buffer.append("\n\t\tname: " + getName());
       buffer.append("\n\t\tvalueDesc: " + myValueDesc );
       buffer.append("\n\t\tvalues: " + getValues());
   
       return buffer.toString();
    }
}