/**
 * Last Modified by: $Author: auszok $
 * On: $Date: 2005/11/28 15:32:56 $
 */
 
package kaos.core.csi.extension;

import java.util.Vector;
import java.util.List;

/**
 * Defulat implementaion of the KAoSExtensionComponent interface
 * 
 * $Revision: 1.1 $
 */
public class KAoSExtensionComponentImpl implements KAoSExtensionComponent
{
	private Vector _associatedOntTypes;
	private String _componentName;
	private boolean _enabledStatus = true;

	/**
     * Basic Constructor
     * 
	 * @param name - name of the component
     * 
	 * @param ontTypes - list of ontology concept names characterizing the component functionality
	 */
	public KAoSExtensionComponentImpl(String name, Vector ontTypes)
	{
		_associatedOntTypes = ontTypes;
		_componentName = name;
	}

    /**
     * Simplified Constructor
     * 
     * @param name - name of the component
     * 
     * @param ontType - single ontology concept name characterizing the component functionality
     */
    public KAoSExtensionComponentImpl(String name, String ontType)
	{
	    _associatedOntTypes = new Vector(); 
		_associatedOntTypes.add(ontType);
		_componentName = name;
	}

    /**
     * Returns the associated ontological concepts for the KAoSExtensionComponent object.
     *      
     * @return  List containing names of the concepts the KAoSExtensionComponent is associated with.
     */
	public List getOntologicalAttributes()
	{
	    return _associatedOntTypes;
	}

    /**
     * Return the (unique) name of this KAoSExtensionComponent object.
     *      
     * @return  name of this component.
     */
	public String getName()
	{
	    return _componentName;
	}
	
	/**
     * Set 'true/false', indicating the 'enabled/disabled' status of the component.
     *      
     * @param	status   boolean parameter to set the 'enabled/disabled' status to
     */
	public void setEnabledStatus(boolean status)
	{
	    _enabledStatus = status;
	}
	
	/**
     * Return 'true/false', indicating the 'enabled/disabled' status of the component.
     *      
     * @return	a boolean parameter indicating the 'enabled/disabled' status of 
     *			this component.
     */
	public boolean getEnabledStatus()
	{
	    return _enabledStatus;
	}
}
