/**
 * Last Modified by: $Author: auszok $
 * On: $Date: 2006/01/30 04:14:31 $
 */
 
package kaos.core.csi.extension;

import kaos.core.util.Logger;

import javax.agent.service.ServiceException;
import javax.agent.service.ServiceFailure;
import javax.agent.service.ServiceProperties;

import java.util.Vector;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Enumeration;
import java.util.List;

import java.io.Serializable;

import ri.service.boot.BootProperties;

/**
 * This class implements the interface defines methods to manage a repository of KAoSExtensionComponents
 */
public class KAoSExtensionComponentManagerImpl implements KAoSExtensionComponentManager, Serializable
{
	/**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = 7933284673353135415L;
    
    private String    _repositoryType; // see kaos.core.csi.extension.KAoSExtensionsTypes
    
    private Hashtable _components;
    private BootProperties _componentsClassNames;
    
	private Hashtable _superConcepts;	
   
    /**
     * Constructor
     */
    public KAoSExtensionComponentManagerImpl(String type)
	{
		_repositoryType = type;
		_superConcepts = new Hashtable();
		_components = new Hashtable();
	}
	
    /**
     * @see kaos.core.csi.extension.KAoSExtensionComponentManager#registerComponent(kaos.core.csi.extension.KAoSExtensionComponent)
     */
	public String getRepositoryManagerType()
	{
		return _repositoryType;
	}
	
    /**
     * @see kaos.core.csi.extension.KAoSExtensionComponentManager#registerComponent(kaos.core.csi.extension.KAoSExtensionComponent)
     */
    public boolean registerComponent(KAoSExtensionComponent extensionComponent) 
    {
        _components.put(extensionComponent.getName(), extensionComponent);
        
        for (Iterator associatedOntTypesIT = extensionComponent.getOntologicalAttributes().iterator(); associatedOntTypesIT.hasNext(); )
        {
            String currentAssociatedOntType = (String)associatedOntTypesIT.next();
            
            // Check if the super concepts of the current associatedOntType has already been cached
            HashSet hs = (HashSet)_superConcepts.get(currentAssociatedOntType);
            
            if (hs == null)
                // a new addition; find its super concepts and cache them
                // TODO: go to directory service and get its super concepts; when OntologyInterfaces is avialable in CSI
                hs = new HashSet();             
            
            _superConcepts.put(currentAssociatedOntType, hs); 
        }
       
        return true;
    }
    
    /**
     * @see kaos.core.csi.extension.KAoSExtensionComponentManager#deregisterComponent(java.lang.String)
     */
    public boolean deregisterComponent(String componentName) 
    {
        if (_components.get(componentName) == null)
            return false;
        else
        {
            _components.remove(componentName);
            return true;
        }
    }
    
    /**
     * @see kaos.core.csi.extension.KAoSExtensionComponentManager#getComponent(java.lang.String)
     */
    public KAoSExtensionComponent getComponent(String associatedOntType)
    {        
        if (associatedOntType == null)
            return null;
        
        // check if the given component is already available
        for (Iterator _componentsIT = _components.values().iterator(); _componentsIT.hasNext(); )
        {
            KAoSExtensionComponent currentKAoSExtensionComponent = (KAoSExtensionComponent)_componentsIT.next();
            
            if (currentKAoSExtensionComponent.getOntologicalAttributes().contains(associatedOntType))
                return currentKAoSExtensionComponent;
        }
        
        KAoSExtensionComponent newComponent = null;
        
        //Check if there is a information in the boot file about the Java class implementing the needed component
        if (_componentsClassNames.containsKey(associatedOntType) )
        {
            String className = (String) _componentsClassNames.get(associatedOntType);
            
            try 
            {
                 printDebugString(".getComponent: tries to instantiate component using class - " +  className, Logger.INFO);
                 
                 Class componentClass = null;
               
                 try 
                 {
                     componentClass = Class.forName(className);
                 }
                 catch (java.lang.ClassNotFoundException cnfe) 
                 {
                     // This is what will happen if the class cannot be found
                     printDebugString(".getComponent: Unable to find component class - " + className, Logger.WARN);
                     
                     throw cnfe;
                 }
                     
                 try 
                 {
                     newComponent = (KAoSExtensionComponent)componentClass.newInstance();
                 }
                 catch(Exception e) 
                 {
                     printDebugString(".getComponent: error when creating a new instance of " + componentClass.getName() + " " + e, Logger.WARN);
                 }
                 
                 _components.put(newComponent.getName(), newComponent);
            }
            catch(Exception e) 
            {
                printDebugString(".getComponent: error when creating a new instance- see " + e, Logger.WARN);
            }
        }
        
        // If creation of the component was not sucessful try to find an existing component responsible for handling more generic ontology type
        if ((HashSet)_superConcepts.get(associatedOntType) == null) 
            return newComponent;
        else
            for (Iterator _superConceptsIT = ((HashSet)_superConcepts.get(associatedOntType)).iterator(); _superConceptsIT.hasNext(); )
            {
                String currentSuperConcept = (String)_superConceptsIT.next();
            
                newComponent = getComponent(currentSuperConcept);
            
                if (newComponent != null) 
                    return newComponent;
            }
        
        return newComponent;
    }
	

    /**
     * @see kaos.core.csi.extension.KAoSExtensionComponentManager#changeComponentEnabledStatus(java.lang.String, boolean)
     */
	public void changeComponentEnabledStatus(String associatedOntType, boolean state)
	{
		KAoSExtensionComponent temp = (KAoSExtensionComponent)_components.get(associatedOntType);
				
		if (temp == null) return;
		
		temp.setEnabledStatus(state);
	}
	
    /**
     * @see kaos.core.csi.extension.KAoSExtensionComponentManager#getComponentEnabledStatus(java.lang.String)
     */
	public boolean getComponentEnabledStatus(String associatedOntType)
	{
		KAoSExtensionComponent temp = (KAoSExtensionComponent)_components.get(associatedOntType);
				
		if(temp == null) return false;
		
		return temp.getEnabledStatus();
	}
	
    /**
     * @see kaos.core.csi.extension.KAoSExtensionComponentManager#getAllKAoSExtensionComponents()
     */
	public Vector getAllKAoSExtensionComponents()
	{
		Vector ret = new Vector();
		
		for (Enumeration myEnum = _components.elements(); myEnum.hasMoreElements(); )
			ret.addElement(myEnum.nextElement());
		
		return ret;
	}
	
    /**
     * @see kaos.core.csi.extension.KAoSExtensionComponentManager#removeAllKAoSExtensionComponents()
     */
	public void removeAllKAoSExtensionComponents()
	{
		_components = new Hashtable();
	}
	
    /**
     * @see kaos.core.csi.extension.KAoSExtensionComponentManager#getRepositoryTableInformation()
     */
	public Hashtable getRepositoryTableInformation()
	{
		Hashtable ret = new Hashtable();
		
		if(_components.size() == 0)
			return ret;
		
		for (Enumeration values = _components.elements(); values.hasMoreElements(); )
		{			
			KAoSExtensionComponent kec = (KAoSExtensionComponent)values.nextElement();
			List ontConcepts= kec.getOntologicalAttributes();
            
			if ( ontConcepts == null || ontConcepts.size() == 0 ) 
                continue;
			
            for (Iterator it = ontConcepts.iterator(); it.hasNext(); ) 
                ret.put((String) it.next(), kec.getName());
		}
		
		return ret;
	}  

    /*****************************************************************************************
    javax.agent.service.Service interface implementation  - begin
    *****************************************************************************************/
  
    /**
     * Obtain the properties of the service.
     *
     * @return      the properties of the service.
     */
    public ServiceProperties getServiceProperties() throws ServiceException, ServiceFailure
    {
        return null;
    }

    /**
     * Set the properties of the service.
     *
     * @param props    containing a set of the properties of the service.
     */
    public void setServiceProperties (ServiceProperties props) throws ServiceException, ServiceFailure
    {
    }

    public void initialize(ServiceProperties env) throws ServiceException
    {
        if (env == null) return;
      
        printDebugString("\n\n " + getRepositoryManagerType() + "ManagerImpl.initialize: received: env : " + env, Logger.INFO);
      
        // Read information about the Java classes implementing components from the boot file; associated with the javax.agent.service key
        _componentsClassNames = (BootProperties) env.get("javax.agent.service." + getRepositoryManagerType());  
     
        printDebugString("\n\n " + getRepositoryManagerType() + 
                "ManagerImpl.initialize: components information in the boot file -\n" + _componentsClassNames, Logger.INFO);
    }
  
    /*****************************************************************************************
    javax.agent.service.Service interface implementation  - end
    *****************************************************************************************/
    
    /////////////////////////////   PRIVATE METHODS BEGIN
    
    /**
     * A utility method to print a String.  Add the name of the class printing the String.
     * 
     * @param msg           The String to be printed.
     */ 
    protected static void  printDebugString (String msg, int loggerDisplayLevel) 
    {    
        _logger.logMessage("**** InstanceClassifierManagerImpl: \n                " + msg, loggerDisplayLevel);
    }

    private static Logger _logger = Logger.getLogger(KAoSExtensionComponentManagerImpl.class);
}