/**
 * Last Modified by: $Author: skulkarni $
 * On: $Date: 2004/11/22 15:56:35 $
 */
 
package kaos.core.csi.observe;

/**
 * Defines the operations to register/deregister and notify KAoSObservers interested in updates when
 * the state of this KAoSObservable changes or an event occurs that is associated with the KAoSObservable.
 * 
 * @author KAoS Team
 * $Revision: 1.2 $
 */
public interface KAoSObservable
{
   /** 
    * Register the given KAoSObserver with this KAoSObservable for updates
    * described by the updateType and updateConstraints.
    *
    * @param updateType         String indicating type of update the KAoSObserver is interested in.
    *
    * @param updateCondition    Object describing the conditions under which to send the update. 
    *
    * @param observer           KAoSObserver registering with for updates. 
    *
    */
   public void registerObserver(String updateType, 
                                Object updateCondition,
                                KAoSObserver observer);
                                
   /** 
    * Deregister the given KAoSObserver for the given update type from the registry of observers associated
    * with this KAoSObservable.
    *
    * @param updateType         String indicating type of update the KAoSObserver is no longer interested in.
    *
    * @param observer           KAoSObserver deregistering. 
    *
    */
   public void deregisterObserver(String updateType, 
								  Object updateCondition,
								  KAoSObserver observer);
   
   /** 
    * Notify observers interested in the given updateType wiht the given update object.
    *
    * @param updateType         String describing the type of update to perform. 
    *
    * @param update             Object containing the update.  
    *
    */
   public void notify(String updateType, 
					  Object updateCondition,
					  Object update);
 }
