/*
 * KAoSObservableImpl.java
 *
 * Created on March 4, 2005, 5:52 PM
 */

package kaos.core.csi.observe;

import javax.agent.service.transport.MessageTransportService;
import javax.agent.Locator;
import java.util.*;
import kaos.core.util.Logger;
import kaos.core.csi.AgentCapabilityImpl;
import kaos.core.service.directory.KAoSAgentDescription;

/**
 *
 * @author  jlott
 */
public class KAoSObservableImpl implements KAoSObservable
{
    private MessageTransportService _mts;
    private Locator _locator;
    Hashtable _observers = new Hashtable();
    private Logger _logger = Logger.getLogger(KAoSObservableImpl.class.getName());
    private String _name;
    
    /** Creates a new instance of KAoSObservableImpl */
    public KAoSObservableImpl (MessageTransportService mts,
                               Locator senderLocator,
                               String senderName)
    {
        _mts = mts;
        _locator = senderLocator;
        _name = senderName;
    }

	public synchronized void registerObserver (String updateType, 
                                               Object updateCondition,
                                               KAoSObserver observer)
	{	   	     
        _logger.logMessage("registerObserver: updateType = " + updateType, Logger.DEBUG);
		if ( observer instanceof RemoteKAoSObserver ) 
		{
			RemoteKAoSObserver rObserver = (RemoteKAoSObserver)observer;
			rObserver.setTransportInfo(_mts,
				_locator,
				_name);
		}	   						
		
		_logger.logMessage("registerObserver: registering observer for condition" + updateCondition, Logger.DEBUG);
	   
		//register observer	    				 
		if (!(_observers.containsKey(updateType))) 
		{
			_observers.put(updateType, new Hashtable());   
		}
        Hashtable observersForType = (Hashtable) _observers.get(updateType);
        
		if (!(observersForType.containsKey(updateCondition))) 
		{
			observersForType.put(updateCondition, new Vector());   
		}
		
		Vector observersForCondition = (Vector) observersForType.get(updateCondition) ;
		observersForCondition.add(observer);
		
		_logger.logMessage("updateObservers: added observer:\n " + observer + "\n for: " + updateCondition
			+ "\n now observer vector: " + observersForCondition, Logger.DEBUG);
	}
                                
	/** 
	 * Deregister the given KAoSObserver for the given update type from the registry of observers associated
	 * with this KAoSObservable.
	 *
	 * @param updateType         String indicating type of update the KAoSObserver is no longer interested in.
	 *							 For now the DS would handle only KAoSConstants.REGISTRATION_UPDATE.
	 * 
	 * @param updateCondition	 See the same in the method heading for registerObserver. 	
	 *
	 * @param observer           the KAoSObserver deregistering. 
	 *
	 */
	public void deregisterObserver(String updateType, 
                                   Object updateCondition,
                                   KAoSObserver observer)
	{	
        synchronized (_observers) {
            if (_observers.containsKey(updateType)) {
                Hashtable observersForType = (Hashtable) _observers.get(updateType);
                if (observersForType.containsKey(updateCondition))  
                {
                    Vector observersForCondition = (Vector) observersForType.get(updateCondition);
                    if (!observersForCondition.remove(observer)) {
                        _logger.logMessage("deregisterObserver: observer not registered " + observer, Logger.WARN);
                    }
                }
                else {
                    _logger.logMessage("deregisterObserver: no observers for condition " + updateCondition, Logger.WARN);
                }
            }
            else {
                _logger.logMessage("deregisterObserver: no observers for type " + updateType, Logger.WARN);
            }
        }
	}
   
	/** 
	 * Notify observers interested in the given updateType wiht the given update object.
	 *
	 * @param updateType         String describing the type of update to perform. 
	 *							 For now the DS would handle only KAoSConstants.REGISTRATION_UPDATE.
	 * 
	 * @param updateCondition	 See the same in the method heading for registerObserver. 	
	 *
	 * @param update             Object containing the update. 
	 *							 For now this is the agent description of the matching agent. Please note that
	 *							 if the observer has registered for 'AGENT_LIST', then it will initially get a list
	 *                           of all agent descriptions, and afterwards will keep getting individual descriptions
	 *                           as new agents come online.
	 *                           If it has not registered for AGENT_LIST, then it will descriptions of all matching
	 *                           agents seperately (i.e. not all in one list) and then will keep getting individual 
	 *							 descriptions as new agents come online.
	 *                           
	 *                           
	 *
	 */
	public void notify (String updateType, 
                        Object updateCondition,
                        Object update)
	{	   
	   
		_logger.logMessage(".notify: sending update type: " + updateType + "\n  updateCondition: " + updateCondition +
			"\n Update value: " + update, Logger.INFO);		 
	   	   
        synchronized (_observers) {
            if (_observers.containsKey(updateType)) {
                Hashtable observersForType = (Hashtable) _observers.get(updateType);
                Iterator it = observersForType.keySet().iterator();
                while (it.hasNext()) {
                    Hashtable condition = (Hashtable) it.next();
                    if (conditionsMatch(condition, (Hashtable) updateCondition)) {                        
                        Vector observers = (Vector) observersForType.get(condition);
                        Enumeration observersEnum = observers.elements();

                        while ( observersEnum.hasMoreElements() ) 
                        {
                            KAoSObserver observer = (KAoSObserver)observersEnum.nextElement();
                            _logger.logMessage(".notify: sending update to : " + observer
                                                + " observer condition: " + condition, Logger.INFO);					 

                            try
                            {
                                observer.update(_name, updateType, condition, update);
                                _logger.logMessage(".notify: finished sending update", Logger.DEBUG);
                            }
                            catch(Exception e)
                            {
                                e.printStackTrace();
                            }
                        }
                    }
                }
            }
            else {
                _logger.logMessage(".notify: there is no observer for updateType: " + updateType, Logger.INFO);
            }
        }
	}	
    
    private boolean conditionsMatch (Hashtable requestedCondition, Hashtable actualCondition)
    {
        boolean conditionsMatch = true;
        Enumeration keys = requestedCondition.keys();
        while(keys.hasMoreElements())
        {
            String key = (String)keys.nextElement();

            if (actualCondition.containsKey(key))
            {
                // the key exists in the description
                Object actualConditionValue = actualCondition.get(key);
                Object requestedConditionValue = requestedCondition.get(key);
                if ( key.equals(KAoSAgentDescription.AGENT_CAPABILITIES) ) {
                    if ( (actualConditionValue instanceof List) &&  (requestedConditionValue instanceof List) )  {
                        if ( ! AgentCapabilityImpl.capabilitiesMatch((List) requestedConditionValue, (List) actualConditionValue )  ) {
                            conditionsMatch = false;
                            break;
                        }
                    }
                }
                else if (!requestedConditionValue.equals(actualConditionValue)) {
                    conditionsMatch = false;
                    break;
                }
            }
            else {
                conditionsMatch = false;
                break;
            }
        }
        return conditionsMatch;
    }
        
}
