/**
 * Last Modified by: $Author: rjeffers $
 * On: $Date: 2005/11/23 17:49:07 $
 */
 
package kaos.core.csi.policy;

import kaos.core.util.PolicyMsg;
import javax.agent.service.ServiceFailure;

import java.util.Vector;

/**
 * Defines the operations to manage policies in KAoS
 * including adding, modifying and removing.
 * 
 * @author KAoS Team
 */
public interface PolicyManagement 
{
    /**
     * Add the specified policy into KAoS.
     *
     * @param  owlPolicyDescription	    OWL String containing the policy description.
     *
     * @throws                          ServiceFailure if the policy service is not available.
     */
    public void addPolicy (String owlPolicyDescription) 
        throws ServiceFailure;
    
    /**
     * Modify the specified policy in KAoS.
     *
     * @param  policyID				    String containing the unique policy id.
     * 
     * @param  owlPolicyDescription     OWL String containing the modified policy description.
     *
     * @throws                          ServiceFailure if the policy service is not available.
     */
    public void modifyPolicy (String policyID, String owlPolicyDescription) 
        throws ServiceFailure;

    /**
     * Remove the specified policy from KAoS.
     *
     * @param  policyID				String containing the unique policy id.
     *
     * @throws                      ServiceFailure if the policy service is not available.
     */
    public void removePolicy (String policyID) 
        throws ServiceFailure;
    /**
     * Return the policy identified by the given id.
     * 
     * @param id      String containing the id of the requested policy.
     * 
     * @return PolicyMsg representing.
     */
    public PolicyMsg getPolicy (String id) 
    	throws ServiceFailure;
    
    /**
     * Return policies for the entity, whose id is specified.
     * 
     * @param id      String containing the id of the entity, whose policies are being requested.
     * 
     * @return Vector of policies.
     */
    public Vector getPoliciesForEntity (String id) 
    	throws ServiceFailure;
}
