package kaos.core.csi.policy;

import javax.agent.service.ServiceFailure;

import java.util.HashMap;
import java.util.Vector;

import kaos.core.service.boot.KAoSBootstrapper;
import kaos.core.service.directory.KAoSAgentDirectoryServiceProxy;
import kaos.core.util.AttributeMsg;
import kaos.core.util.KAoSConstants;
import kaos.core.util.Msg;
import kaos.core.util.PolicyMsg;
import kaos.core.util.SubjectMsg;
import kaos.core.util.UniqueIdentifier;
import kaos.kpat.applet.ClassBuilderUtils;
import kaos.core.csi.OntPropertyDescription;
import kaos.core.csi.OntPropertyDescriptionImpl;
import kaos.core.csi.OntInstanceDescription;
import kaos.core.csi.OntInstanceDescriptionImpl;
import kaos.ontology.util.KAoSClassBuilderImpl;
import kaos.ontology.vocabulary.ActionConcepts;
import kaos.ontology.vocabulary.ActorConcepts;
import kaos.ontology.vocabulary.PlaceConcepts;
import kaos.ontology.vocabulary.DefaultNamespaces;
import kaos.ontology.vocabulary.PolicyConcepts;
import kaos.ontology.vocabulary.PolicyConstants;
import kaos.spatial.vocabulary.PhysicalPlaceConcepts;
import kaos.spatial.vocabulary.PhysicalAddressConcepts;
import kaos.robots.ontology.RobotActionConcepts;

import kaos.policy.information.OntologyPolicyContainer;
import kaos.policy.util.KAoSPolicyBuilderImpl;
import kaos.policy.obligation.ObligationConstraintsBuilder;


/**
 * Defines the operations to manage policies in KAoS
 * including adding, modifying and removing.
 * 
 * @author KAoS Team
 */
public class PolicyManagementImpl
	implements PolicyManagement
{
	private static final PolicyManagementImpl   __INSTANCE = new PolicyManagementImpl() ;
    private static boolean                      __INITIALIZED = false;
        
    private KAoSAgentDirectoryServiceProxy      _dirProxy = null;
    private HashMap								_policies = new HashMap();
	//private Logger                              _logger = Logger.getLogger(this);
	/**
     * Retrieve an instance of PolicyChecking.  The returned instance has initialized communication with the DS.
     *
     * @param transportName String specifying the name of the transport to use for communication with the KAoS Directory Service.
     * 
     * @return a singleton, PolicyChecking.
     */
    public static PolicyManagement getInstance(String transportName)
    throws Exception
    {
        if ( __INITIALIZED ) return __INSTANCE;
        
        __INSTANCE.initialize(transportName);
        __INITIALIZED = true;
        return __INSTANCE;
    }
    
    /**
     * The method adds the specified policy into KAoS.
     *
     * @param  policy				OWL String containing the policy description.
     *
     * @throws                      ServiceFailure if the policy service is not available.
     */
//    public void addPolicy (String owlPolicyDescription) 
//        throws ServiceFailure
//	{
//		
//	}
	
    /**
     * The method adds the specified policy into KAoS.
     *
     * @param  policyID				pilicy id
     *
     * @throws                      ServiceFailure if the policy service is not available.
     */
    public void addPolicy (String policyID) 
        throws ServiceFailure
	{
		PolicyMsg policyMsg = (PolicyMsg)_policies.get(policyID);
		Vector addedPolicies = new Vector();
		addedPolicies.addElement(policyMsg);
		try{
			_dirProxy.updatePolicies(addedPolicies,
									 new Vector(),
									 new Vector()); 
			_policies.put(policyID, policyMsg);
		}
		catch(Exception e){
			if (e instanceof ServiceFailure){
				throw (ServiceFailure) e;
			}
			else
			{
				e.printStackTrace();
			}
		}
	}

    
    /**
     * The method modifies the specified policy in KAoS.
     *
     * @param  policyID				String containing the unique policy id.
     * 
     * @param  owlPolicyDescription	OWL String containing the modified policy description.
     *
     * @throws                      ServiceFailure if the policy service is not available.
     */
    public void modifyPolicy (String policyID, String owlPolicyDescription) 
        throws ServiceFailure
	{
		
	}

    /**
     * The method removes the specified policy from KAoS.
     *
     * @param  policyID				String containing the unique policy id.
     *
     * @throws                      ServiceFailure if the policy service is not available.
     */
    public void removePolicy (String policyID) 
        throws ServiceFailure
	{
		PolicyMsg policyMsg = (PolicyMsg)_policies.get(policyID);
		Vector removedPolicies = new Vector();
		removedPolicies.addElement(policyMsg);
		try{
			_dirProxy.updatePolicies(new Vector(),
									 new Vector(),
									 removedPolicies); 			
		}
		catch(Exception e){
			if (e instanceof ServiceFailure){
				throw (ServiceFailure)e;
			}
			else
			{
				e.printStackTrace();
			}
		}
	}
    
    /**
     * Return policies for the entity, whose id is specified.
     * 
     * @param id      String containing the id of the entity, whose policies are being requested.
     * 
     * @return Vector of policies.
     */
    public Vector getPoliciesForEntity (String id) 
    throws ServiceFailure
    {
    	if ( _dirProxy != null ) {
            return  _dirProxy.getPoliciesForEntity(id);
         }
    	throw new ServiceFailure("Policy Service unavailable! Unable to obtain policies for: " + id);
    }
    
    /**
     * Return the policy identified by the given id.
     * 
     * @param id      String containing the id of the requested policy.
     * 
     * @return PolicyMsg representing.
     */
    public PolicyMsg getPolicy (String id) 
    	throws ServiceFailure
    {
    	//TODO - IMPLEMENT THE METHOD!
    	return new PolicyMsg(new Msg());
    	
    }
	//policy creation methods  //////////////////////////////////////////
	/**
	 * Creates and commits an XYZCoordinateAddress instance
	 * 
	 * @return	String of XYZCoordinateAddress name
	 */
	public OntInstanceDescription createXYZCoordinateAddress(String instanceName, double x, double y)
	{
        OntInstanceDescription address = null;
        
		try{
            address = new OntInstanceDescriptionImpl(instanceName, PhysicalAddressConcepts.XYZCoordinateAddress());
		      
            OntPropertyDescription xCoordinate = new OntPropertyDescriptionImpl(PhysicalAddressConcepts.XCoordinate());
            xCoordinate.addValue(new Double(x).toString());
            address.addProperty(xCoordinate);
            
            OntPropertyDescription yCoordinate = new OntPropertyDescriptionImpl(PhysicalAddressConcepts.YCoordinate());
            yCoordinate.addValue(new Double(y).toString());
            address.addProperty(yCoordinate);
            
			// commit the instance
			_dirProxy.loadOntology(address.getInstanceOntModel(), true);
		}
		catch (Exception e){
			e.printStackTrace();
		}
		
		return address;	
    }
		
	public OntInstanceDescription createPoint(String instanceName, double x, double y)
	{
        OntInstanceDescription point = null;
        
		try{
            point = new OntInstanceDescriptionImpl(instanceName, PhysicalPlaceConcepts.Point());
              
            OntPropertyDescription pointAddress = new OntPropertyDescriptionImpl(PlaceConcepts.hasAddress());
            pointAddress.addValue(instanceName + "_xyzAddress");
            pointAddress.setValueDescForValue(instanceName + "_xyzAddress", createXYZCoordinateAddress(instanceName + "_xyzAddress", x, y));
            point.addProperty(pointAddress);
            
			// commit the instance
            _dirProxy.loadOntology(point.getInstanceOntModel(), true);
		}
		catch (Exception e){
			e.printStackTrace();
		}
		
		return point;
	}
	
	/**
	 * Creates and commits an area instance
	 * 
	 * @return	String of area name
	 */
	public String createArea(double ulx, double uly, double lrx, double lry)
	{
		String fullName = DefaultNamespaces.defaultKAoSInstancesNamespace + UniqueIdentifier.GenerateUID();
		try{
			// create the Area instance 
            OntInstanceDescription area = new OntInstanceDescriptionImpl(fullName, PhysicalPlaceConcepts.Area());

			//create the upper left point and add to area
			String upperLeftPointName = DefaultNamespaces.defaultKAoSInstancesNamespace + UniqueIdentifier.GenerateUID();
            OntInstanceDescription upperLeftPointDsc = createPoint(upperLeftPointName, ulx, uly);
            OntPropertyDescription upperLeftPoint = new OntPropertyDescriptionImpl(PhysicalPlaceConcepts.hasCornerPoint());
            upperLeftPoint.addValue(upperLeftPointName);
            upperLeftPoint.setValueDescForValue(upperLeftPointName, upperLeftPointDsc);
            area.addProperty(upperLeftPoint);
            
			//create the lower right point and add to area
            String lowerRightPointName = DefaultNamespaces.defaultKAoSInstancesNamespace + UniqueIdentifier.GenerateUID();
            OntInstanceDescription lowerRightPointDsc = createPoint(lowerRightPointName, lrx, lry);
            OntPropertyDescription lowerRightPoint = new OntPropertyDescriptionImpl(PhysicalPlaceConcepts.hasOppositeCornerPoint());
            lowerRightPoint.addValue(lowerRightPointName);
            lowerRightPoint.setValueDescForValue(lowerRightPointName, lowerRightPointDsc);
            area.addProperty(lowerRightPoint);
		
			// commit the instance
			_dirProxy.loadOntology(area.getInstanceOntModel(),true);
		}
		catch (Exception e){
			e.printStackTrace();
		}	
		
		return fullName;
	}
								   
	public String createPolicy(String policyName,
							 String policyDescription,
							 int policyPriority,
							 String subjectScope,
							 String subjectID,
							 String subjectName,
							 String action,
							 String modality)
	{
		//create policy id
		String policyID = ClassBuilderUtils.POLICY_ID_PREFIX + UniqueIdentifier.GenerateUID();

		try{
			// create the controls action class
			KAoSClassBuilderImpl controls = new KAoSClassBuilderImpl(
					ClassBuilderUtils.generateClassNameForPolicy (action,
																  DefaultNamespaces.controlActionPostfix,
																  policyID));
																		   
			controls.addImmediateBaseClass(action);

			// create the damlPolicy
			KAoSPolicyBuilderImpl damlPolicyBuilder = new KAoSPolicyBuilderImpl();
			// Set Id and Modality
			damlPolicyBuilder.setPolicyIDAndModalityType(policyID,
														 PolicyConcepts.NegAuthorizationPolicy());
			// Set Policy Name
			damlPolicyBuilder.setPolicyName(policyName);
			// Set Priority
			damlPolicyBuilder.setPriority(policyPriority);
			// Set Actor Class or Agent Instance
			if (subjectScope.equals(KAoSConstants.ACTOR_CLASS_SCOPE))
			{
				controls.setPropertyRangeClass(ActionConcepts.performedBy(),
			                                       subjectID);
			}
			else {
				controls.addPropertyRangeInstance(ActionConcepts.performedBy(),
			                                          subjectID);
			}

			// add the controls action to the damlPolicy
			damlPolicyBuilder.setControlsActionClass(controls);
			OntologyPolicyContainer damlPolicy = damlPolicyBuilder.getPolicy();

			// create the PolicyMsg and add the damlPolicy as an attribute
			SubjectMsg subjectMsg = new SubjectMsg(subjectID,
												   subjectName,
												   subjectScope);
			Vector subjects = new Vector();
			subjects.addElement(subjectMsg);  
			PolicyMsg policyMsg = new PolicyMsg(policyID,
												policyName,
												policyDescription,
												action,
												"", // admin
												subjects,
												true);
			policyMsg.setModality(modality);
			policyMsg.setPriority(Integer.toString(policyPriority));
			policyMsg.setAttribute(new AttributeMsg(AttributeMsg.ONTOLOGY_CONTENT,
			                                            damlPolicy,
			                                            true));
			_policies.put(policyID, policyMsg);
		}
		catch(Exception e){
			e.printStackTrace();
		}
		return policyID;
	}
	
	public String createObligation(String policyName,
							 String policyDescription,
							 int policyPriority,
							 String subjectScope,
							 String subjectID,
							 String subjectName,
							 String obligation,
							 String modality,
							 String triggerAction)
	{
		//create policy id
		String policyID = ClassBuilderUtils.POLICY_ID_PREFIX + UniqueIdentifier.GenerateUID();

		try{
			// create the trigger action class
			KAoSClassBuilderImpl trigger = new KAoSClassBuilderImpl(
					ClassBuilderUtils.generateClassNameForPolicy (triggerAction,
																  DefaultNamespaces.controlActionPostfix,
																  policyID));
																		   
			trigger.addImmediateBaseClass(triggerAction);

			// create the controls action class
			KAoSClassBuilderImpl controls = new KAoSClassBuilderImpl(
					ClassBuilderUtils.generateClassNameForPolicy (obligation,
																  DefaultNamespaces.controlActionPostfix,
																  policyID));
																		   
			controls.addImmediateBaseClass(obligation);

			// create the damlPolicy
			KAoSPolicyBuilderImpl damlPolicyBuilder = new KAoSPolicyBuilderImpl();
			// Set Id and Modality
			damlPolicyBuilder.setPolicyIDAndModalityType(policyID,
														 PolicyConcepts.PosObligationPolicy());
			// Set Policy Name
			damlPolicyBuilder.setPolicyName(policyName);
			// Set Priority
			damlPolicyBuilder.setPriority(policyPriority);
			// Set Actor Class or Agent Instance for obligation
			if (subjectScope.equals(KAoSConstants.ACTOR_CLASS_SCOPE))
			{
				controls.setPropertyRangeClass(ActionConcepts.performedBy(),
			                                       subjectID);
			}
			else {
				controls.addPropertyRangeInstance(ActionConcepts.performedBy(),
			                                          subjectID);
			}

			// Set Actor Class or Agent Instance for trigger
			if (subjectScope.equals(KAoSConstants.ACTOR_CLASS_SCOPE))
			{
				trigger.setPropertyRangeClass(ActionConcepts.performedBy(),
			                                       subjectID);
			}
			else {
				trigger.addPropertyRangeInstance(ActionConcepts.performedBy(),
			                                          subjectID);
			}
			//add trigger
			damlPolicyBuilder.setTriggerActionClass(trigger);
			
			//ad obligation constraint
			String baseClass = PolicyConcepts.FinishBeforeRefActionStart();
            String className = ClassBuilderUtils.generateClassNameForPolicy(baseClass,
                                                                            DefaultNamespaces.obligationConstraintPostfix,
                                                                            policyID);            
            ObligationConstraintsBuilder builder = new ObligationConstraintsBuilder(className);
            try {
                builder.addImmediateBaseClass(baseClass);
            }
            catch (Exception xcp) {
                xcp.printStackTrace();
            }
			damlPolicyBuilder.setObligationConstraints(builder);

			// add the obligation
			damlPolicyBuilder.setControlsActionClass(controls);
			OntologyPolicyContainer damlPolicy = damlPolicyBuilder.getPolicy();

			// create the PolicyMsg and add the damlPolicy as an attribute
			SubjectMsg subjectMsg = new SubjectMsg(subjectID,
												   subjectName,
												   subjectScope);
			Vector subjects = new Vector();
			subjects.addElement(subjectMsg);  
			PolicyMsg policyMsg = new PolicyMsg(policyID,
												policyName,
												policyDescription,
												triggerAction,
												"", // admin
												subjects,
												true);
			policyMsg.setModality(modality);
			policyMsg.setPriority(Integer.toString(policyPriority));
			policyMsg.setAttribute(new AttributeMsg(AttributeMsg.ONTOLOGY_CONTENT,
			                                            damlPolicy,
			                                            true));
			_policies.put(policyID, policyMsg);
		
		}
		catch(Exception e){
			e.printStackTrace();
		}
		return policyID;
	}
	
	public String createRestrictedAreaPolicy(String policyName,
								 String policyDescription,
								 int policyPriority,
								 String subjectScope,
								 String subjectID,
								 String subjectName,
								 double ulx,
								 double uly,
								 double lrx,
								 double lry)
	{
		String action = RobotActionConcepts.MoveTo();		
		String modality = PolicyConcepts.NegAuthorizationPolicy();
		
		//create policy id
		String policyID = ClassBuilderUtils.POLICY_ID_PREFIX + UniqueIdentifier.GenerateUID();

		try{
			// create the controls action class
			KAoSClassBuilderImpl controls = new KAoSClassBuilderImpl(
					ClassBuilderUtils.generateClassNameForPolicy (action,
																  DefaultNamespaces.controlActionPostfix,
																  policyID));
																		   
			controls.addImmediateBaseClass(action);

			//create an area
			String areaName = createArea(ulx, uly, lrx,lry);		
		
			// add properties to the action
			// example of adding a property with an instance value
			controls.addPropertyRangeInstance(ActionConcepts.movingTo(),
											  areaName,
											  PolicyConstants._toClassRestriction);

			// create the damlPolicy
			KAoSPolicyBuilderImpl damlPolicyBuilder = new KAoSPolicyBuilderImpl();
			// Set Id and Modality
			damlPolicyBuilder.setPolicyIDAndModalityType(policyID,
														 modality);
			// Set Policy Name
			damlPolicyBuilder.setPolicyName(policyName);
			// Set Priority
			damlPolicyBuilder.setPriority(policyPriority);
			// Set Actor Class or Agent Instance
			if (subjectScope.equals(KAoSConstants.ACTOR_CLASS_SCOPE))
			{
				controls.setPropertyRangeClass(ActionConcepts.performedBy(),
			                                       subjectID);
			}
			else {
				controls.addPropertyRangeInstance(ActionConcepts.performedBy(),
			                                          subjectID);
			}

			// add the controls action to the damlPolicy
			damlPolicyBuilder.setControlsActionClass(controls);
			OntologyPolicyContainer damlPolicy = damlPolicyBuilder.getPolicy();

			// create the PolicyMsg and add the damlPolicy as an attribute
			SubjectMsg subjectMsg = new SubjectMsg(subjectID,
												   subjectName,
												   subjectScope);
			Vector subjects = new Vector();
			subjects.addElement(subjectMsg);  
			PolicyMsg policyMsg = new PolicyMsg(policyID,
												policyName,
												policyDescription,
												action,
												"", // admin
												subjects,
												true);
			policyMsg.setModality(modality);
			policyMsg.setPriority(Integer.toString(policyPriority));
			policyMsg.setAttribute(new AttributeMsg(AttributeMsg.ONTOLOGY_CONTENT,
			                                            damlPolicy,
			                                            true));
			_policies.put(policyID, policyMsg);
		
		}
		catch(Exception e){
			e.printStackTrace();
		}
		return policyID;
	}
	
	public void initialize(String transportName)
		throws Exception
    {
        try {
            _dirProxy = KAoSBootstrapper.getKAoSDirectoryProxyForTransport(transportName);
        }
        catch(Exception e) {
            e.printStackTrace();
            throw new Exception("    \n\n    *** RegistrationImpl: unable to obtain a reference the DirectoryService for transport: "
                                + transportName + "!");
        }
    }
	
	public static void main(String[] args)
	{
		PolicyManagementImpl pmi = new PolicyManagementImpl();
		System.out.println("initialize");		
		try{
			pmi.initialize("tcp");
		}
		catch(Exception e){
			e.printStackTrace();
		}
		
		System.out.println("creating policy");		
		String policyName = "myTestPolicy";
		String policyDescription = "This is a test policy";
		int policyPriority = 1;
		
		String subjectScope = KAoSConstants.ACTOR_CLASS_SCOPE;
		String subjectID = ActorConcepts.Robot();
		String subjectName = ActorConcepts.Robot();
/*
		String policyID = pmi.createPolicy(policyName,
										   policyDescription,
										   policyPriority,
										   subjectScope,
										   subjectID,
										   subjectName,
										   action,
										   modality);
		
		System.out.println("commit policy");	
		try{
			pmi.addPolicy(policyID);
		}
		catch(Exception e){
			e.printStackTrace();
		}
		
		System.out.println("done");	
		
		System.out.println("creating point");	
		String instanceName = "point=(2,4)";
		pmi.createXYZCoordinateAddress(instanceName, 2,4);
		
		System.out.println("creating area");
		pmi.createArea(3, 6, 6, 3);
		
		//System.out.println("creating area");	
		
		try{
			Thread.sleep(15000);
		}
		catch(Exception e2){
			e2.printStackTrace();
		}
		System.out.println("removing policy");		
		try{
			pmi.removePolicy(policyID);
		}
		catch(Exception e){
			e.printStackTrace();
		}
*/		
		System.out.println("creating RA");
		policyName = "myTestRA";
		policyDescription = "This is a test RA";
		policyPriority = 1;
		
		subjectScope = KAoSConstants.ACTOR_CLASS_SCOPE;
		subjectID = ActorConcepts.Robot();
		subjectName = ActorConcepts.Robot();

		String policyID2 = pmi.createRestrictedAreaPolicy(policyName,
															policyDescription,
															policyPriority,
															subjectScope,
															subjectID,
															subjectName,
															5,10,10,5);
		System.out.println("commit policy");	
		try{
			pmi.addPolicy(policyID2);
		}
		catch(Exception e){
			e.printStackTrace();
		}
		
		System.out.println("done");
	}
}
