/**
 * Last Modified by: $Author: rjeffers $
 * On: $Date: 2005/10/31 21:45:39 $
 */
 
package kaos.core.csi.policy.enforcement;

import kaos.core.csi.policy.KAoSSecurityException;

import javax.agent.service.ServiceFailure;

/**
 *
 * This interface defines methods for an enforcer enforcing CommunicationAction.
 *
 * @author KAoS Team
 * $Revision: 1.3 $
 */
public interface CommunicationActionEnforcer
    extends Enforcer
{	
    /**
     * Authorize a communication action described by properties expressed by the arguments to this method.
     * Authorization is based on the current set of policies stored in the Guard.
     * All arguments, except for the dsTransportName, contain the values of properties with the same name
     * defined in the ontology for CommunicationAction.
     *
     * @param performedBy           String containing the name of the actor performing the CommunicationAction.
     *
     * @param hasDestination        String containing the name of the entity receiving the CommunicationAction.
     *
     * @param carriesMessage        String containing the content of the CommunicationAction.
     *
     * @param hasPartner            String containing the name of the partner in the CommunicationAction.
     *
     * @param dsTransportName       String containing the name of the transport used by KAoS Policy service
     *                              and other KAoS services to communicate with each other.
     *
     * @throws KAoSSecurityException    when the CommunicationAction is forbidden by the current policies.
     *
     * @throws NullPointerException     when performedBy is null.  
     *
     * @throws ServiceFailure           when the Policy service cannot be obtained, or fails durin action authorization.
     */
    public void authorizeCommunicationAction(String performedBy, 
                                             String hasDestination,
                                             String carriesMessage,
                                             String hasPartner,
                                             String dsTransportName)
    throws KAoSSecurityException, NullPointerException, ServiceFailure;
 }
