/**
 * Last Modified by: $Author: rjeffers $
 * On: $Date: 2005/10/28 16:58:57 $
 */
 
package kaos.core.csi.query;

import kaos.ontology.util.QueryFailure;
import kaos.ontology.management.UnknownConceptException;
import javax.agent.service.directory.DirectoryFailure;
import java.util.Set;

/**
 * Defines the operations to query the different aspect of state stored in the Directory Service 
 * 
 * @author KAoS Team
 * $Revision: 1.3 $
 */
public interface QueryState
{
    /**
     * Test if the <a href="http://www-ksl.stanford.edu/knowledge-sharing/kif/">KIF</a> statemant is true according to the ontologies in the ontology repository.
     *
     * @param statement        to be tested for truth
     *
     * @return                 true if the statement is true and false in the opposite case
     * 
     * @throws                 DirectoryFailure if the connection to the DS cannot be established
     *
     * @throws                 QueryFailure if query failed
     */
    public boolean testTrue (String statement) throws DirectoryFailure, QueryFailure;
    
    /**
     * Get values of the selected variable satisfing the <a href="http://www-ksl.stanford.edu/knowledge-sharing/kif/">KIF</a> 
     * query with <b>possibly many variable</b>.
     *
     * @param variableName             the name of the selected variable
     * 
     * @param multiplyVariablesQuery   the query
     *
     * @return                         a set with values satisfing the query
     *
     * @throws                 DirectoryFailure if the connection to the DS cannot be established
     *
     * @throws                         QueryFailure if query failed.
     */
    public Set askQuery (String variableName, String multiplyVariablesQuery) throws DirectoryFailure, QueryFailure;
    
    /**
     * Get set of instances of the given concept; if the concept is an instance itself the set contains just this instance.
     *
     * @param conceptName       The name of the concept
     *
     * @return                  Set of instance ids
     *
     * @throws                  DirectoryFailure if the connection to the DS cannot be established
     * 
     * @exception               UnknownConceptException if conceptName is unknow
     */
    public Set getInstancesOf (String conceptName) throws DirectoryFailure, UnknownConceptException;
    
    /**
     * Get a single value of the variable satisfing the <a href="http://www-ksl.stanford.edu/knowledge-sharing/kif/">KIF</a> 
     *                                                  query: <b>(property subject ?val)</b>.
     *
     * @param property         String containing the name of the property.
     *
     * @param subject          String containing the name of the subject.
     *
     * @return                 String containing the single value for the property on this subject.
     *
     * @throws                 DirectoryFailure if the connection to the DS cannot be established.
     *
     * @throws                 QueryFailure if reasoning failed.
     *
     */
    public String getUniqueValueForProperty (String property, String subject) throws DirectoryFailure, QueryFailure;
    
    
 }
