/**
 * Last Modified by: $Author: rjeffers $
 * On: $Date: 2005/01/06 03:00:00 $
 */
 
package kaos.core.csi.registration;

import java.io.Serializable;

/**
 * Contains the modification type, its succes/failure flag and the description and error code for the modification failure.
 *
 * @author KAoS Team
 * $Revision: 1.5 $
 */
public interface ModificationResult
    extends Serializable
{
    public static final String ADD = "0";
    public static final String ADD_ELEMENT_TO_LIST = "1";
    public static final String ADD_ELEMENTS_TO_LIST = "2";
    public static final String REPLACE = "3";
    public static final String DELETE = "4";
    public static final String DELETE_ELEMENT_FROM_LIST = "5";
    public static final String DELETE_ELEMENTS_FROM_LIST = "6";
    public static final String ADD_ELEMENT_TO_TABLE = "7";
    public static final String ADD_ELEMENTS_TO_TABLE = "8";
    public static final String DELETE_ELEMENT_FROM_TABLE = "9";
    public static final String DELETE_ELEMENTS_FROM_TABLE = "10";
    
    /** 
     *  Returns a String specifying the type of modification.  The possible values are:
     * 
     *  ADD -   a new key-value pair to the properties Hashtable.  The key is the name of the property, the value is the 
     *          value of the property.
     *
     *  ADD_ELEMENT_TO_LIST - add the specified property value to the existing property, whose value is a List,  
     *                        for example, to add a domain name to the existing List of domain names.
     *
     *  ADD_ELEMENTS_TO_LIST - to add the specified property value (which is a List) to the existing property, 
     *                        whose value is a List , for example, to add a List of new domain names to the existing List
     *                        of domain names. 
     *                       
     *  REPLACE - to replace a value of the existing key-value pair with the specified property value.
     *
     *  DELETE - delete the key-value pair from the properties Hashtable.
     *
     *  DELETE_ELEMENT_FROM_LIST - to delete the specified property value from the existing property, whose value is a List,
     *                            for example, to remove an ontological type from the existing List of types.
     *                           
     *
     *  DELETE_ELEMENTS_FROM_LIST - to remove the specified property value (which is a List) from the existing property,
     *                            whose value is a List , for example, to remove a List of ontological types from
     *                            the existing List of types.
     *
     *  ADD_ELEMENT_TO_TABLE - add the specified property value ( a 1-element Hashtable)to the existing property, whose value is a Table,  
     *                        for example, to add an ontological type and its description to the existing Table 
     *                        of types and their descriptions.
     *
     *  ADD_ELEMENTS_TO_TABLE - to add the specified property value (which is a Hashtable)) to the existing property, 
     *                        whose value is a Hashtable , for example, to add a Hashtable of new ontological types and their descriptions
     *                        to the existing Hashtable of types and their descriptions.
     *                        
     *  DELETE_ELEMENT_FROM_TABLE - to delete the specified property value from the existing property, whose value is a Hashtable,
     *                            for example, to remove an ontological type and its description from the existing Hashtable of types.
     *                           
     *
     *  DELETE_ELEMENTS_FROM_TABLE - to remove the specified property value, which is a Collection of keys to remove from 
     *                            the existing property, whose value is a Hashtable, for example, to remove a Collection of ontological types from
     *                            the existing Hashtable of types->descriptions.
     *                            
     
     *
     * @return      String describing the type of the modification.
     */
     
    
    public String  getModificationType();
   
    /** 
     * Set the type of the modification. 
     *
     * @param type      String containing the type of modification.
     */
    public void  setModificationType(String type);
    
    /** 
     * Set the success/failure status of the modification. 
     *
     * @param flag      boolean indicating modification success/failure; true means success.
     */
    public void  setSuccess (boolean flag);
   
    /**
     * Obtain the success/failure status of the modification.
     *
     * @return boolean  indicating modification success/failure; true means success
     */
    public boolean  isSuccess();
    
    /** 
     * Return the description of modification failure. 
     *
     * @return  String   describing modification failure.
     */
    public String  getModificationFailureDescription();
   
    /**
     * Store the description of modification failure. 
     *
     * @param desc  String containing the description of modification failure.
     */
    public void  setModificationFailureDescription(String desc);
    
    /** 
     * Return the error code of the modification failure. 
     *
     * @return  int   describing modification failure.
     */
    public int  getModificationFailureCode();
   
    /**
     * Store the error code of the modification failure. 
     *
     * @param code  int containing the error code of the modification failure. 
     */
    public void  setModificationFailureCode(int code);

}
