/**
 * Last Modified by: $Author: mjohnson $
 * On: $Date: 2005/10/11 16:33:30 $
 */
 
package kaos.core.csi.request;

import java.util.Calendar;

/**
 *
 * @author KAoS Team
 * $Revision: 1.1 $
 */
public interface FeedbackReceiver
{
   /** 
    * This method is invoked when the request is accepted by the robot.
    *
    * @param time           Calendar time of acceptance.
    *
    * @param acceptInfo     Object containing additional (optional) info about the acceptance.
    *
    * @param actionID       String containing the id of the requested action/information.
    *
    * @param robotID        String containing the name of the robot performing the action.
    *                       
    */
   public void  onAccepted(Calendar time, Object acceptInfo, String actionID, String robotID); 
   
   /** 
    * This method is invoked when the request is refused by the robot.
    *
    * @param time           Calendar time of refusal.
    *
    * @param reason         Object containing the reason for refusing the request.
    *
    * @param actionID       String containing the id of the requested action/information.
    *
    * @param robotID        String containing the name of the robot performing the action.
    */
   public void onRefused(Calendar time, Object reason, String actionID, String robotID);
   
   /** 
    * This method is invoked upon completion of the request byt the requested robot. 
    *
    * @param time           Calendar time of information/action completion.
    *
    * @param result         Object containing the result of the requested action. 
    *
    * @param actionID       String containing the id of the requested action/information.
    *
    * @param robotID        String containing the name of the robot performing the action.
    *
    */
   public void onCompleted(Calendar time, Object result, String actionID, String robotID);
   
   /** 
    * This method is invoked when the robot starts the requested action.
    *
    * @param time           Calendar time of starting an action.
    *
    * @param startedInfo    Object containing additional (optional) info about the start of the action.
    *
    * @param actionID       String containing the id of the requested action/information.
    *
    * @param robotID        String containing the name of the robot performing the action.
    *
    *                       
    */
   public void onStarted(Calendar time, Object startedInfo, String actionID, String robotID);
   
   /** 
    * This method is invoked when the requested action is aborted.
    *
    * @param time           Calendar time of aborting an action.
    *
    * @param reason         Object containing the reason for aborting an action.
    *
    * @param actionID       String containing the id of the requested action/information.
    *
    * @param robotID        String containing the name of the robot performing the action.
    *
    */
   public void onAborted(Calendar time, Object reason, String actionID, String robotID);
   
   
   /** 
    * Reset callback settings. 
    *
    */
  // public void reset();
       
   
   /**
     * Receive a status message describing the execution of the requested action. 
     * There maybe several status messages sent during action execution.
     *
     * @param statusMsg     Object containing the result of the requested action/information.
     *
     * @param actionID      String containing the id of the requested action/information.
     *
     * @param robotID       String containing the name of the robot performing the action.
     *
     */                                     
   public void receiveKAoSStatus(String statusMsg, String actionID, String robotID); 
      
}
