/**
 * Last Modified by: $Author: rjeffers $
 * On: $Date: 2005/12/09 14:50:59 $
 */
package kaos.core.csi.request;

import javax.agent.service.transport.NotLocatableException;
import javax.agent.service.transport.TransportFailure;

import kaos.core.csi.ActionInstanceDescription;
import kaos.core.service.directory.KAoSEntityDescription;

/**
 * Defines the operations to send an obligation request described by an obligation ActionInstanceDescription and
 * trigger ActionInstanceDescription to an entity for execution.  The entity is identified either by a unique ID or a KAoSEntityDescription.
 * 
 * @author KAoS team
 *
 */
public interface ObligationRequestManager {
	
	/**
	 * 
	 * This method sends an obligation request to an entity identified by the given receiverDesc to execute an obligation action described  
	 * by the given obligation ActionInstanceDescription and the trigger ActionInstanceDescription.
	 *  
	 * @param	    receiverDesc	        is a KAosEntityDescription of the entity, which is requested
	 *								        to execute the obligation request.
	 * 
	 * @param       obligationAID		 	is an ActionInstanceDescription containing the obligation action name
	 * 										and its properties.
	 * 
	 * @param		triggerAID	            is an ActionInstanceDescription containing the trigger action name
	 * 										and its properties. 
	 * 
	 * @throws	    ActionFailureException  containing the reason for failure.  
	 *				                        See ontology to determine specific reasons available for 
	 *				                        a given action
	 *
	 * @throws	    NotLocatableException   when the entity reqested to perform the action cannot be located.
	 *				                        
	 * @throws	    TransportFailure        when the request cannot be sent due to transport failure.
	 * 
	 **/
	public void sendObligationRequest(KAoSEntityDescription receiverDesc,
									  ActionInstanceDescription obligationAID,
									  ActionInstanceDescription triggerAID)
		throws ActionFailureException, NotLocatableException, TransportFailure;
    
    /**
     * 
	 * This method sends an obligation request to an entity identified by its unique ID to execute an obligation action described  
	 * by the given obligation ActionInstanceDescription and the trigger ActionInstanceDescription.
	 *
	 * @param	    receiverID	            is the String that identifies the entity, which is  
	 *							            requested to perform the action.  The receiverID is the one
	 *							            that the action executor has registered with KAoS.
	 *
	 * @param       obligationAID		 	is an ActionInstanceDescription containing the obligation action name
	 * 										and its properties.
	 * 
	 * @param		triggerAID	            is an ActionInstanceDescription containing the trigger action name
	 * 										and its properties. 
	 * 
	 * @throws	    ActionFailureException  containing the reason for failure.  
	 *				                        See ontology to determine specific reasons available for 
	 *				                        a given action
	 *
	 * @throws	    NotLocatableException   when the entity reqested to perform the action cannot be located.
	 *				                        
	 * @throws	    TransportFailure        when the request cannot be sent due to transport failure.
	 * 
	 **/
	public void sendObligationRequest(String receiverID,
									  ActionInstanceDescription obligationAID,
									  ActionInstanceDescription triggerAID)
		throws ActionFailureException, NotLocatableException, TransportFailure;;


}
