/**
 * Last Modified by: $Author: auszok $
 * On: $Date: 2005/11/28 15:32:56 $
 */
package kaos.core.csi.usecase;

import kaos.core.csi.CSIFactory;
import kaos.core.csi.query.QueryRegistration;
import kaos.core.csi.registration.Registration;
import kaos.core.csi.transport.TransportSupport;
import kaos.core.service.directory.KAoSAgentDescription;
import kaos.core.util.MsgUtils;

import javax.agent.TransportMessage;
import javax.agent.service.directory.DirectoryFailure;
import javax.agent.service.transport.MessageListener;
import javax.agent.service.transport.NotLocatableException;
import javax.agent.service.transport.TransportFailure;
import javax.agent.service.directory.SearchException;

/**
 * $Revision: 1.3 $
 */
public class SimpleSender
    implements  MessageListener
    
 {   
    /**
        This class llustrates an implementation of a SimpleSender: an entity capable of sending  
        messages.
        
        Configuration data includes: nickname, names of kaos domains it will register in, name of a transport 
        to use for communication within KAoS.
        
        Bootstrapping steps:
	        - obtains a reference to CSI Registration
	        - binds itself to CSI Transport 
	        - asks CSI Registration  to create a KAoSAgentDescription for  itself
	        - asks CSI Registration to register with KAoS

	        
        Running
	        - looks up another agent, whose nameis given
	        - sends a message to it
    */
    protected KAoSAgentDescription      _agentDescription = null;
    protected TransportSupport          _transportSupport;
    protected QueryRegistration         _query;
    
    public SimpleSender( )
    {
    }

    
    //////////////////////////////////////////////////////////////

    public void initialize(String name, KAoSAgentDescription agentDescription, String transportName)
    throws Exception
    {
       
        _agentDescription = agentDescription;
                                                                    
        _transportSupport = CSIFactory.getTransportSupport();
        
        _transportSupport.bindMsgListenerToTransport(this, name, agentDescription);
        
        CSIFactory.setCurrentTransport(transportName);
        _query = CSIFactory.getQueryRegistration();
       
    }
    
    //////////////////////////////////////////////////////////////
    // Implementation of the MessageListener interface

    /**
     * Implements the receiveMessage method mandated by the
     * MessageListener interface.  This method prints
     * the incoming TransportMessage to System.out.
     *
     */
    public void receiveMessage( TransportMessage tmsg )
    {
        System.out.println( "\n   *** SimpleSender: received transport message:\n" + tmsg);
        Object msgContent = MsgUtils.unwrapObjectContent(tmsg);

        if ( msgContent == null ) {
            System.out.println("\n   *** SimpleSender: \n receiveMessage: could not retrieve the content from: " + tmsg);
            return;
        }
                    
       String senderName = MsgUtils.getSender(tmsg);
       
       System.out.println("\n   *** SimpleSender: received the following content from " + senderName + ": " + msgContent);
    }

    /**
     * Two agents are equal if their respective hash codes
     * are equal.
     *
     * @return true if obj equals this SimpleSender.
     */
    public boolean equals( Object obj )
    {
        if ( obj == this )
            return true;

        if ( !(obj instanceof SimpleSender) )
            return false;

        return this.hashCode() == ((SimpleSender)obj).hashCode();
    }

    /**
     * Determines the hash code for the SimpleSender.  The SimpleSender
     * hash code is simply the hash code of the represented agents
     * AgentName.  If the AgentName field has not yet been set,
     * then the hash code is found from <code>super.hashCode()
     * </code>.
     *
     * @return int the hashCode for this SimpleSender.
     *
     */
    public int hashCode()
    {
        return _agentDescription.getEntityNameAsString() == null ? super.hashCode() : _agentDescription.getEntityNameAsString().hashCode();
    }

    //////////////////////////////////////////////////////////////
    
    public void sendMessageTo(String receiverName)
        throws DirectoryFailure, SearchException, NotLocatableException, TransportFailure
    {
        KAoSAgentDescription receiverDesc = _query.getActor(receiverName);
        System.out.println("\n   *** SimpleSender: Requested description for: " +  receiverName 
                          +"\n                     Received: \n\n    " + receiverDesc);
        if ( receiverDesc == null ) return;
        _transportSupport.sendMessageTo(_agentDescription,
                                        receiverDesc,
                                        "\n\n   Hello World!");
       
       System.out.println("\n   *** SimpleSender: Sent the following message to " +  receiverName + ": \n\n   Hello World!");
    }
    
   //////////////////////////////////////////////////////////////
   // Entry point...

   public static void main(String [] args)
      throws Exception      
    {
        String sender = null;
        String transport = null;
        String receiver = null;
        if (args.length > 0) {
	        int i = 0;
	        while (i < args.length) {
	            if (args[i].equalsIgnoreCase ("-sender")) {
	                if ((i+1) < args.length) {
    	                sender = args[i+1];
    	                i++;
    	            }
    	            else {
    	                System.out.println ("Must specify Entity Name with -sender");
    	                System.exit(0);
    	            }
	            }
	            else if (args[i].equalsIgnoreCase ("-receiver")) {
	                if ((i+1) < args.length) {
    	                receiver = args[i+1];
    	                i++;
    	            }
    	            else {
    	                System.out.println ("Must specify  transport with -transport.  Valid transports: corba, grid, tcp.");
    	                System.exit(0);
    	            }
	            }
	            else if (args[i].equalsIgnoreCase ("-transport")) {
	                if ((i+1) < args.length) {
    	                transport = args[i+1];
    	                i++;
    	            }
    	            else {
    	                System.out.println ("Must specify  transport with -transport.  Valid transports: corba, grid, tcp.");
    	                System.exit(0);
    	            }
	            }
                 i++;
	        }// while
	    }
	    
        if ( sender == null) {
            System.out.println ("\n     *** Must specify Sender Name with -sender\n");
            return;
        }
        
        if ( receiver == null) {
            System.out.println ("\n     *** Must specify Receiver Name with -receiver\n");
            return;
        }
        
        if ( transport == null) {
            System.out.println ("\n     *** Must specify  transport with -transport.  Valid transports: corba, grid, tcp.");
            return;
        }
        
        System.out.println ("\n     <>  Sender name                 " + sender);
        System.out.println ("\n     <>  Receiver name               " + receiver);
	    System.out.println ("\n     <>  Requested transport:        " + transport);
        
        Registration registration = null;
        try {
            CSIFactory.setCurrentTransport(transport);
            registration = CSIFactory.getRegistration();
        }
        catch(Exception e){
            e.printStackTrace();
            System.out.println ("  **** Unable to obtain CSI services: Registration. Exiting...");
            return;
        }
        
        // Ask Registration to create a KAoSAgentDescription for the sender, so it can bind to the transport.
        KAoSAgentDescription agentDescription = registration.createKAoSAgentDescription(sender,       // use nickname as guid
                                                                                        sender,       // nickname
                                                                                        null,         // no domains to register in
                                                                                        transport,
                                                                                        null);      // no extra transport info necessary
                                                                    
        
        SimpleSender usecase = new SimpleSender();
        
        try {
            usecase.initialize(sender, agentDescription, transport);
        }
        catch(Exception e) {
            e.printStackTrace();
            return;
        }
       
        usecase.sendMessageTo(receiver);
       
        synchronized ( Thread.currentThread() ) {
         Thread.currentThread().wait();

      }

   }
}

