/**
 * Last Modified by: $Author: rjeffers $
 * On: $Date: 2005/12/16 05:59:15 $ 
 */
package kaos.core.csi.usecase.legacy;

import java.net.URISyntaxException;
import java.util.Vector;

import kaos.core.csi.ActionInstanceDescription;
import kaos.core.csi.ActionInstanceDescriptionImpl;
import kaos.core.csi.policy.enforcement.EnforcerImpl;
import kaos.core.csi.policy.KAoSSecurityException;
import kaos.core.csi.usecase.legacy.vocabulary.LegacySystemActionConcepts;

import javax.agent.service.ServiceFailure;

/**
 * This class illustrates how a KAoS Enforcer can enforce selected actions
 * in a legacy system.  When a given action has to be executed according
 * to the policies, the Enforcer asks KAoS policy service for action authorization
 * and, if permitted, delegates action execution to the legacy system.
 * Not all actions are constrained by policies. Some actions by design are just delegated
 * for execution to the legacy system without being authorized. 
 * 
 * @author KAoS Team
 *
 */
public class LegacySystemEnforcerImpl 
	extends EnforcerImpl
	implements LegacySystem {
	     
	// LegacySystem to delegate to legacy actions
	LegacySystem 				_legacySystem;
	
	/**
	 * @param name		String containing the Enforcer name.
	 * @param ontTypes	Vector containing the ontological names of actions this Enforcer is assocdiated with.
	 */
	public LegacySystemEnforcerImpl(String name, Vector ontTypes)
	{
		super(name, ontTypes);
	}
	
	/**
	 * @param name		String containing the Enforcer name.
	 * @param ontType	Vector containing the ontological names of actions this Enforcer is assocdiated with.
	 */
	public LegacySystemEnforcerImpl(String name, String ontType) 
	{
		super(name, ontType);
	}
	
	/*****************************************************************************************
	 kaos.core.csi.usecase.legacy.LegacySystem interface implementation - begin
    *****************************************************************************************/
	/**
	 * 
	 * Accepts a request for another action in the legacy system.
	 * 
	 * @param actionRequestorID		String containing the unique ID of the action requestor. 
	 * 
	 * @return boolean indicating success/failure of the action execution.
	 */
	public boolean anotherLegacyAction(String actionRequestorID)
	{
		if ( _legacySystem != null ) {
			return _legacySystem.anotherLegacyAction(actionRequestorID);
		}
		else {
			return false;
		}
	}
	
	/**
	 * Accepts a request for a legacy action in the legacy system.
	 * 
	 * Perform requested legacy action. Before the action execution, check if the action is allowed
	 * according to the current set of policies.  Note that the action is composed from the business 
	 * perspective:  the actor of the action is the user requesting the action, vs. the software executing the action.
	 * 
	 * @param actionRequestorID		String containing the unique ID of the action requestor. 
	 * 
	 * @return boolean indicating success/failure of the action execution.
	 * 
	 */
	public boolean legacyAction(String actionRequestorID) {
	
		ActionInstanceDescription subscriptionAction = null;
		try{
			subscriptionAction =  new ActionInstanceDescriptionImpl(null, 
																	LegacySystemActionConcepts.LegacySystemAction(),
																    actionRequestorID);
		}
		catch(URISyntaxException e){
			e.printStackTrace();
			if ( _legacySystem != null ) {
				return _legacySystem.legacyAction(actionRequestorID);
			}
			else {
				return false;
			}
		}
        
		try
		{
			_policyChecking.checkPermission(subscriptionAction, null);
		}
		catch (KAoSSecurityException e)
		{
			e.printStackTrace();
			System.out.println("\n\n	***  Policy forbids user identified by:\n\n\t " + actionRequestorID
							   + "\n\n	***  to request an action from the system!!!");
			return false;
		}
		catch(ServiceFailure sf) 
		{
			sf.printStackTrace();
			if ( _legacySystem != null ) {
				return _legacySystem.legacyAction(actionRequestorID);
			}
			else {
				return false;
			}
		}
       
		if ( _legacySystem != null ) {
			return _legacySystem.legacyAction(actionRequestorID);
		}
		else {
			return false;
		}
	}

	/*****************************************************************************************
	 kaos.core.csi.usecase.legacy.LegacySystem interface implementation - end
	 *****************************************************************************************/
	
	/////////////// Public/protected  methods 
	public void setControlledLegacySystem(LegacySystem legacySystem)
	{
		_legacySystem = legacySystem;
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) 
	
	{
	
	}

}
