/**
 * Last Modified by: $Author: rjeffers $
 * On: $Date: 2006/01/27 22:04:30 $
 */

package kaos.core.csi.util.tcpSupport;

import kaos.core.csi.CSIFactory;
import kaos.core.csi.util.xmlSupport.XMLToCSIConstants;
import kaos.core.csi.util.xmlSupport.XMLToCSILink;
import kaos.core.csi.util.xmlSupport.XMLToCSITranslator;
import kaos.core.util.Logger; 

import java.rmi.dgc.VMID;
import java.io.BufferedReader;
import java.net.Socket;

/**
 * This class accepts requests for actions from a client. It reads input stream from the client socket, parses it
 * and formats it into a String that contains request name and properties.  The string is passed to the XML-CSILink
 * for further processing of the request.
 *
 * This class also receives a response from the action executor and sends it back to the client.
 */
public class ServerRequestHandler 
    extends CommHandler
    implements ResponseSender  
{
    private XMLToCSILink    _link; 
    private boolean         _testing = false;
    private long            _timeBegin;
	private String          _id;
	private boolean         _linkInitialized = false;

		
	/**
	 * Constructor
	 * 
	 * @param	clientSocket Socket used to communicate with a client.
	 */
	
	public ServerRequestHandler (Socket clientSocket)
	throws Exception
    {
        super(clientSocket);
		 VMID vmid = new VMID();
        _id = vmid.toString();
    }
    
    /**
     * 
     */
    protected void processBuffer(String line, BufferedReader bufferedReader)
	{
	    long timeEnd = 0;
	    if (_logger.isEnabledFor(Logger.DEBUG)){
			_logger.logMessage("processBuffer: received:\n " + line 
					             + "\n\tLine length:  " + line.length(), Logger.DEBUG);
		}
	    if  ( line.length()== 10 ) {
		    if ( line.equals("START-TEST") ){ 
			    _testing = true;
			    _timeBegin = System.currentTimeMillis();
			    return;
		    }
		    else if ( line.equals("END---TEST")) { 
			    _testing = false;
			    timeEnd = System.currentTimeMillis();
	            System.out.println("\n*** testFileSend: ended test.  Duration: " + (timeEnd - _timeBegin) +"ms\n");
	            _os.println ("ENDED-TEST");
			    return;
		    }
	        else {
			    processLine(line);
			    return;
		    }
	    }// end if   ( line.length()== 10 ) {
					
        String begin = null;
					
		if   ( line.length() > 16 ) {
		    // There are 2 types of headers, indicating whether the xml file content
		    // is going to be transmitted, or just the xml file name.
		    // The last character specifies the transmission type:
		    // header for file content: XML-TRANSMISSION-1
			// header for file name   : XML-TRANSMISSION-0
		    begin = line.substring(0, 17);
	
	        if ( ! ( begin.equals(XMLToCSIConstants.XML_TRANSMISSION_HEADER)) ){
			    processLine(line);
			    return;
	        }
					
		    String xmlTransmissionType = line.substring(17,18); // "1" for file content; "0" for file name

		    if (  xmlTransmissionType.equals( XMLToCSIConstants.XML_TRANSMITTED_FILE_NAME )) {
			    String first = readLine(bufferedReader);    // first line for file name: is the name of the file, including its path        
			    if ( first != null ) {                                             
			        if ( _testing ) {
					    XMLToCSITranslator.getDocumentFromXMLFile(first);
					    return;
					}
				    requestAction(first, xmlTransmissionType);
		        }
			}
		    else if ( xmlTransmissionType.equals( XMLToCSIConstants.XML_TRANSMITTED_FILE_CONTENT )) {
			    String fileContent = getFileContent(bufferedReader);
			    if ( _testing ){
			        XMLToCSITranslator.getDocumentFromXMLString(fileContent);
			        return;
			    }
			    requestAction(fileContent, xmlTransmissionType);
		    }
		    else {
		        processLine(line);
		    }
		}
        else {
	        processLine(line);
        }// end if   ( line.length() > 16 ) {
	}
	
    private void  init()
    {
         _linkInitialized = false;
        try {
            String dsTransport = System.getProperty("agent.msgtransport");
            if ( dsTransport == null ) return;
            System.out.println("\n  ServerRequestHandler: got DS transport: " + dsTransport);
            _link = CSIFactory.getXMLToCSILink(dsTransport); 
            _linkInitialized = true;
        }
        catch(Exception e){
            e.printStackTrace();
        }
    }
    
    /**
     *  Process client request described by the xml string.  The xmlTransmissionType
     *  indicates whether the xml String contains the name of the xml file
     *  or the file content.
     *
     */
    private void requestAction(String xmlString, String xmlTransmissionType) 
    {
        if (_logger.isEnabledFor(Logger.INFO)){
			_logger.logMessage("requestAction: received:\n " + xmlString
			                     + "\n  transmissionType (0 = file name, 1 = filecontent): "  + xmlTransmissionType, Logger.INFO);
		}
        
        if ( ! _linkInitialized ) init();
        
        if  ( _linkInitialized )
        {
            try {
                _link.requestAction(this,xmlString, xmlTransmissionType);
            }
            catch(Exception e){
                e.printStackTrace();
            }
        }
    }
    
    /**
     * Read the received input stream into a string, containing an xml file.
     */
    private String getFileContent(BufferedReader bufferedReader) 
    {
        String endOfFile = XMLToCSIConstants.ACTIONS_END;
	    StringBuffer buffer = new StringBuffer(); 
	                    	                   	                        
	    boolean end = false;
	    String next;

	    while ( ! end ) {
	        if ( ( next = readLine(bufferedReader)) != null ){
	        	if (_logger.isEnabledFor(Logger.DEBUG)){
	    			_logger.logMessage("getFileContent: next line:\n " + next, Logger.DEBUG);
	    		}
	        	if ( next.trim().substring(0, 1).equals("<") ) {
	        		if ( next.indexOf("?xml") == -1 ) {
	        			buffer.append(next + "\n");
	        		}
	        	}
	            if ( next.equals(endOfFile) ){
	                end = true;
	            }
	        }
	        else {
	            end = true; 
	        }
	    }// end while
        if (_logger.isEnabledFor(Logger.INFO)){
			_logger.logMessage("getFileContent: returning content:\n " + buffer.toString(), Logger.INFO);
		}
	                    	                
	    return buffer.toString();
    } 
          
    ///////////////////////////////////////////////////////////////////////  
    //  kaos.core.csi.tcpserver.ResponseSender implementation
    /////////////////////////////////////////////////////////////////////// 
    public synchronized void sendResponse(String response)  
    {
        if (_logger.isEnabledFor(Logger.DEBUG)){
			_logger.logMessage("sendResponse: sending a response to client:\n " + response, Logger.DEBUG);
		}
        
       if ( _os != null ) {
            _os.println(response);
            System.out.println("ServerRequestHandler.sendResponse:\n" + response);
       }
       else {
		    _logger.logMessage("sendResponse: Communication to the client disabled! Unable to send the following response:\n " + response, Logger.WARN);
	    }
    }
   
    /** 
     *
     * @return String    containing the id for this ResponseSender.
     *
     */
    public String  getID()
    {
        return _id;
    }
    //  kaos.core.csi.tcpserver.ResponseSender implementation END
 
   
}