/**
 * Last Modified by: $Author: rjeffers $
 * On: $Date: 2006/01/27 22:04:00 $
 */

package kaos.core.csi.util.xmlSupport;

//import org.apache.xerces.parsers.DOMParser;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import java.util.Hashtable;

// for parsing xml files
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder; 
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

//import org.w3c.dom.Document;
import org.xml.sax.InputSource; 
import java.io.StringWriter;
import java.io.StringReader;


/**
 */
public class XMLToCSITranslator {
    
    public  static final String DOMAIN_FOR_ALL_TCP_BASED_PROTOCOL = "TCP-TO-KAoS-Protocol";
    
    
    private static DocumentBuilder _builder = null;
    //private static DOMParser   _parser = new DOMParser();
    static {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        try {
            _builder = factory.newDocumentBuilder();
        }
        catch(Exception e){
        }
    }
 
/**
 *  Based on the given transmission type, process the given String, which can be either a content of an xml file
 *  or a name of an xml file. Use the appropriate method to translate the input into a Hashtable containing entries
 *  with action names and their properties.
 *
 * @param xmlString             String representing either the content or or the name of an xml file.
 *
 * @param xmlTransmissionType   String indicating whether the xmlString is the content or the name of an xml file. 
 *
 * @return Hashtable            containing action names and properties with their values.
 */
public synchronized static Hashtable translateActions(String xmlString, String xmlTransmissionType)
{
    if ( xmlTransmissionType.equals( XMLToCSIConstants.XML_TRANSMITTED_FILE_CONTENT)) {
        return translateActionsFromXMLString(xmlString);
    }
    else {
        return translateActionsFromXMLFile(xmlString);
    }
}

/**
 *  Translate the given content of an xml file into a Hashtable containing entries of
 *  action names and their properties with values.
 
 * An example of the file content can be:
 
 <?xml version="1.0" encoding="UTF-8"?>
 <Actions>
   <Commands>
      <Command Name="MoveTo">
         <Property Name="movingTo">
            <Location XCoordinate="7.0" YCoordinate="9.0"/>
         </Property>
         <Receiver Name="Ivan"/>
      </Command>
   </Commands>
 </Actions>
 *
 *
 * The resulting Hashtable:
 
 {Commands=Commands, MoveTo={movingTo={XCoordinate=7.0, YCoordinate=9.0}, Receiver=Ivan}}
 
 * @param xmlFileContent        String representing the content of an xml file.
 *
 * @return Hashtable            containing action names and properties with their values.
 */
public synchronized  static Hashtable translateActionsFromXMLString(String xmlFileContent)
{
    if ( xmlFileContent == null || _builder == null ){
        System.out.println("\n XMLToCSITranslator.translateActionsFromXMLString: one of the args is null.\n  _builder: " +  _builder
                           + " xmlFileContent: " +  xmlFileContent + "\n   Unable to continue processing!");
        return null;
    }
    System.out.println("\n XMLToCSITranslator.translateActionsFromXMLString: received input:\n " + xmlFileContent);
    // parse file
    Document document = getDocumentFromXMLString(xmlFileContent);
     
    if ( document != null )
        return translateActionFromDoc(document);
    return  null;
}

/**
 *  Read an xml file with the given name and then translate its content into a Hashtable containing entries of
 *  action names and their properties with values.
 *
 * @param xmlFileName String containing the name of an xml file.
 *
 * @return Hashtable containing action names and properties with their values.
 */
public synchronized  static Hashtable translateActionsFromXMLFile(String xmlFileName)
{
    if ( xmlFileName == null || _builder == null ){
        System.out.println("\n XMLToCSITranslator.translateActionsFromXMLFile: one of the args is null.\n  _builder: " +  _builder
                           + " xmlFileName: " +  xmlFileName + "\n   Unable to continue processing!");
        return null;
    }
    System.out.println("\n XMLToCSITranslator.translateActionsFromXMLFile: received input:\n " + xmlFileName);
    // parse file
    Document document = getDocumentFromXMLFile(xmlFileName);
    if ( document != null )
        return translateActionFromDoc(document);
    return  null;
}

public synchronized static Hashtable translateActionFromDoc(Document document)
{
    NodeList elements = document.getElementsByTagName(XMLToCSIConstants.COMMANDS);

    if (elements !=  null && elements.getLength() > 0) {
        // Processs Commands
        return translateCommandsFromDoc(document);
    }
    
    elements = document.getElementsByTagName(XMLToCSIConstants.REGISTRATIONS);

    if (elements !=  null && elements.getLength() > 0) {
        // Processs Registrations
        return translateRegistrationsFromDoc(document);
    }
    
    elements = document.getElementsByTagName(XMLToCSIConstants.AUTHORIZATIONS);

    if (elements !=  null && elements.getLength() > 0) {
        // Processs Registrations
        return translateAuthorizationsFromDoc(document);
    }
    return null;
}

/**
 *  Traverse the given Document to retrieve an entity name and its spatial properties, if present.
 *  Put the name and values of the entity, properties into a Hashtable.
 *
 * @param document      Document containing the registration info to be retrieved and reformatted into the Hashtable.
 *
 * @return Hashtable    containing an entity name, spatial properties (if present).
 */
public synchronized static Hashtable translateRegistrationsFromDoc(Document document) 
{
    if ( document == null ){
        System.out.println("\n XMLToCSITranslator.translateRegistrationsFromDoc: document is null.  Unable to continue processing!");
        return null;
    }    
    Hashtable registrations = new Hashtable();
    // get elements that match Entity element
    NodeList elements = document.getElementsByTagName(XMLToCSIConstants.ENTITY);

    if (elements == null) {
        return null;
    }

    int elementCount = elements.getLength();
    System.out.println("\n Found: " + elementCount + " Entities");
    for (int i = 0; i < elementCount; i++) {
        Element element = (Element)elements.item(i);  //Entity
        
        String entityName = getElementName(element);
    
        if ( entityName == null ) {
            System.out.println("\n Entity Name attribute is null.  Terminating processing registrations... "); 
            continue;
        }
           
        Hashtable locationCoords = null;
        Hashtable orientCoords = null;
        
        // Location retrieval. Location is optional.
        NodeList locationElements = element.getElementsByTagName(XMLToCSIConstants.LOCATION);
        if ( locationElements != null &&  locationElements.getLength() > 0 ) {
            //Get Location element for this Entity
            Element locationElement = (Element)locationElements.item(0);  // there should be only 1 element
            locationCoords = getSpatialProps(locationElement);
        }
                        
        // Orientation retrieval.  Orientation is optional.  If present, Location must be also present.
        if ( (locationCoords != null) && ( ! locationCoords.isEmpty() ) ) {
            Hashtable entityHash = new Hashtable();
            entityHash.put(XMLToCSIConstants.LOCATION,locationCoords);
            //get Orientation element
            NodeList orientationElements = element.getElementsByTagName(XMLToCSIConstants.ORIENTATION);
            if ( orientationElements != null && orientationElements.getLength() > 0 ) {
                Element orientationElement = (Element)orientationElements.item(0);  // there should be only 1 element
                orientCoords = getSpatialProps(orientationElement);
                if ( orientCoords != null && ( ! orientCoords.isEmpty()) ) {
                    entityHash.put(XMLToCSIConstants.ORIENTATION, orientCoords);      
                }
            }
            registrations.put(entityName,entityHash); 
            registrations.put(XMLToCSIConstants.REGISTRATIONS,XMLToCSIConstants.REGISTRATIONS); 
            System.out.println("\n registrations: " + registrations);
            continue;
        }
        
        registrations.put(entityName,entityName); 
        registrations.put(XMLToCSIConstants.REGISTRATIONS,XMLToCSIConstants.REGISTRATIONS); 
        System.out.println("\n registrations: " + registrations);
        
    }//end for (int i = 0; i < elementCount; i++) Entity list
    return registrations;
} 

/**
 *  Traverse the given Document to retrieve a name of the action to authorize and its properties.
 *  Put the action name and properties with their values into a Hashtable.
 *
 * @param document      Document containing the action info to be retrieved and reformatted into the Hashtable.
 *
 * @return Hashtable    containing the name of the action to be authorzied and its properties(if present and the actor name).
 */
public synchronized static Hashtable translateAuthorizationsFromDoc(Document document) {
    
    if ( document == null  ){
        System.out.println("\n XMLToCSITranslator.translateAuthorizationsFromDoc: document is null. Unable to continue processing!");
        return null;
    }
    Hashtable authorizations = new Hashtable();
    // get elements that match Entity element
    NodeList elements = document.getElementsByTagName(XMLToCSIConstants.ACTION);

    if (elements == null) {
        return null;
    }

    int elementCount = elements.getLength();
    System.out.println("\n Found: " + elementCount + " Action");
    for (int i = 0; i < elementCount; i++) {
        Element element = (Element)elements.item(i);  //Action
        
        String actionName = getElementName(element);
        if ( actionName == null ) {
            System.out.println("\n Action Name attribute is null.  Nothing to process...."); 
            continue;
        }
        Hashtable actionHash = new Hashtable();       
        
        // retrieve action properties
        Hashtable propertiesHash = null;
        NodeList propertyElements = element.getElementsByTagName(XMLToCSIConstants.PROPERTY);
        if ( (propertyElements != null) && (propertyElements.getLength() > 0) ) {
        	propertiesHash = new Hashtable();
        	for (int j = 0; j < propertyElements.getLength(); j++) {
                Element property = (Element)propertyElements.item(j);  //Property element
                String propertyName = getElementName(property);          
                NodeList valueElements = property.getElementsByTagName(XMLToCSIConstants.VALUE);
                if ( (valueElements != null) && (valueElements.getLength() > 0) ) {
                	Element valueElement = (Element)valueElements.item(0);  // there should be only 1 Value element
                	String valueName = getElementName(valueElement);
                	propertiesHash.put(propertyName,valueName);
                } // end if ( (valueElements != null) 
        	} // end for (int j = 0; j < propertyElements.getLength(); j++)
         }// end if ( (propertyElements != null) - check for movingTo property
        
        if ( propertiesHash != null )
        	actionHash.put(XMLToCSIConstants.PROPERTIES,propertiesHash);
        
        /// retrieve actor performing the action
        NodeList actorElements = element.getElementsByTagName(XMLToCSIConstants.ACTOR);
        if ( (actorElements != null) && (actorElements.getLength() > 0) ) {
           Element actorElement = (Element)actorElements.item(0);  // there should be only 1 actor element 
           String actorName = getElementName(actorElement);
           
           if ( actorName != null )
        	   actionHash.put(XMLToCSIConstants.ACTOR,actorName);
        	   
        }// end ( (actorElements != null)
        authorizations.put(actionName,actionHash);
    }  // end int i = 0; i < elementCount; i++)
    
    Hashtable authorizationsHash = new Hashtable();
    authorizationsHash.put(XMLToCSIConstants.AUTHORIZATIONS,authorizations);
    return authorizationsHash;
} 

/**
 *  Traverse the given Document to retrieve a command name and its properties.
 *  Put the command name and properties with their values into a Hashtable.
 *
 * @param document      Document containing the command info to be retrieved and reformatted into the Hashtable.
 *
 * @return Hashtable    containing an entity name, spatial properties (if present).
 */
public synchronized static Hashtable translateCommandsFromDoc(Document document) {
    
    if ( document == null  ){
        System.out.println("\n XMLToCSITranslator.translateCommandsFromDoc: document is null. Unable to continue processing!");
        return null;
    }
    Hashtable commands = new Hashtable();
    // get elements that match Entity element
    NodeList elements = document.getElementsByTagName(XMLToCSIConstants.COMMAND);

    if (elements == null) {
        return null;
    }

    int elementCount = elements.getLength();
    System.out.println("\n Found: " + elementCount + " Command");
    for (int i = 0; i < elementCount; i++) {
        Element element = (Element)elements.item(i);  //Command
        
        String commandName = getElementName(element);
        if ( commandName == null ) {
            System.out.println("\n Command Name attribute is null.  Terminating...."); 
            continue;
        }
        
        /////// MoveTo command
        if ( commandName.equals(XMLToCSIConstants.MOVE_TO) ) {
            Hashtable locationCoords = null;
            
            // get the movingTo property
            NodeList propertyElements = element.getElementsByTagName(XMLToCSIConstants.PROPERTY);
            if ( (propertyElements != null) && (propertyElements.getLength() > 0) ) {
                Element movingToElement = (Element)propertyElements.item(0);  // there should be only 1 movingTo element
                // NamedNodeMap movingToAttributes = movingToElement.getAttributes();
                //get Location element for the movingTo property
                
                NodeList locationElements = movingToElement.getElementsByTagName(XMLToCSIConstants.LOCATION);
                if ( (locationElements != null) && (locationElements.getLength() > 0) ) {
                    Element locationElement = (Element)locationElements.item(0);  // there should be only 1 Location element
                    locationCoords = getSpatialProps(locationElement);
                } // end if ( (locationElements != null) 
             }// end if ( (propertyElements != null) - check for movingTo property
             
             // get the Receiver name
             String receiverName = null;
             
             NodeList receiverElements = element.getElementsByTagName(XMLToCSIConstants.RECEIVER);
             if ( (receiverElements != null) && (receiverElements.getLength() > 0) ) {
                Element receiverElement = (Element)receiverElements.item(0);  // there should be only 1 Receiver element 
                receiverName = getElementName(receiverElement);
             }// end if ( (receiverElements != null) 
                                    
             if ( locationCoords != null && receiverName != null  ) {
                Hashtable moveTo = new Hashtable();
                moveTo.put(XMLToCSIConstants.MOVING_TO,locationCoords);
                moveTo.put(XMLToCSIConstants.RECEIVER,receiverName);
                commands.put(XMLToCSIConstants.MOVE_TO,moveTo);
                commands.put(XMLToCSIConstants.COMMANDS,XMLToCSIConstants.COMMANDS); 
                System.out.println("\n commands: " + commands);
                return commands;
             }
        }//end if ( commandName.equals(XMLToCSIConstants.MOVE_TO)
        /////// End MoveTo command
        
        /////// GetPose command
        if ( commandName.equals(XMLToCSIConstants.GET_POSE) ) {
            // get the Receiver name
            String receiverName = null;
            NodeList receiverElements = element.getElementsByTagName(XMLToCSIConstants.RECEIVER);
            if ( (receiverElements != null) && (receiverElements.getLength() > 0) ) {
                Element receiverElement = (Element)receiverElements.item(0);  // there should be only 1 Receiver element  
                receiverName = getElementName(receiverElement);
             }// end if ( (receiverElements != null) 
                                     
            if (  receiverName != null  ) {
                Hashtable getPose = new Hashtable();
                getPose.put(XMLToCSIConstants.RECEIVER,receiverName);
                commands.put(XMLToCSIConstants.GET_POSE,getPose);
                commands.put(XMLToCSIConstants.COMMANDS,XMLToCSIConstants.COMMANDS); 
                System.out.println("\n commands: " + commands);
                return commands;
             }
        }//end if ( commandName.equals(XMLToCSIConstants.GET_POSE)
        /////// End getPose command
    }  
    return commands;
} 

protected static void print(Element element, NamedNodeMap attributes) {

        System.out.print('<');
        System.out.print(element.getNodeName());
        if (attributes != null) {
            int attributeCount = attributes.getLength();
            for (int i = 0; i < attributeCount; i++) {
                Attr attribute = (Attr)attributes.item(i);
                System.out.print(' ');
                System.out.print(attribute.getNodeName());
                System.out.print("=\"");
                System.out.print(attribute.getNodeValue());
                System.out.print('"');
            }
        }
        System.out.println('>');

} // print(Element,NamedNodeMap)

private static String getElementName(Element element)
{
    NamedNodeMap attributes = element.getAttributes();
    print(element,attributes);
    String name =  null;   
    if (attributes != null && attributes.getLength() > 0 ) {
        int attributeCount = attributes.getLength();
        for (int j = 0; j < attributeCount; j++) {
            Attr attribute = (Attr)attributes.item(j);
            if ( attribute.getNodeName().equals(XMLToCSIConstants.NAME) ) {
                name = attribute.getNodeValue();
                System.out.println("\n Element Name: " + name);
                break;  
            }
        }
    }// end if (attributes != null)  
    return name; 
}

private static Hashtable getSpatialProps(Element element)
{
    Hashtable props = null;
    NamedNodeMap attributes = element.getAttributes();
    print(element,attributes);
    if (attributes != null) {
        int attributeCount = attributes.getLength();
        props = new Hashtable();
        for (int j = 0; j < attributeCount; j++) {
            Attr attribute = (Attr)attributes.item(j);
            props.put(attribute.getNodeName(),attribute.getNodeValue());
        }
        System.out.println("\n Spatial props hash: " + props);
    }// end attributes 
    return props;
}

/**
 *  Read an xml file with the given name, parse it and create a Document representing the content of the file.
 *
 * @param xmlFileName   String containing the name of the xml file to be read and parsed.
 *
 * @return Document     representing the content of the parsed file.
 */
public static Document getDocumentFromXMLFile(String xmlFileName)
{
    Document document = null;

    try {
        // _parser.parse(xmlFileName);  - uses xerces parser
        // document = _parser.getDocument();
        InputSource inputSource = new InputSource(xmlFileName);
        document = _builder.parse(inputSource);
    }
    catch (SAXParseException e) {
        // ignore
    }
    catch (Exception e) {
        System.err.println("error: Parse error occurred - "+e.getMessage());
        if (e instanceof SAXException) {
            Exception nested = ((SAXException)e).getException();
            if (nested != null) {
                e = nested;
            }
        }
        e.printStackTrace(System.err);
    }
    return document;
 }
 
/**
 *  Parse the given String representing the content of an xml file it and create a Document based on the content.
 *
 * @param xmlString     String representing the content of an xml file to be parsed.
 *
 * @return Document     representing the content of the parsed file.
 */
 public static Document getDocumentFromXMLString(String xmlString)
 {
    Document document = null;
    try{
        InputSource inputSource = new InputSource();
        inputSource.setCharacterStream(new StringReader(xmlString));
        document = _builder.parse(inputSource);
    }
    catch (SAXParseException e) {
        // ignore
    }
    catch (Exception e) {
        System.err.println("error: Parse error occurred - "+e.getMessage());
        if (e instanceof SAXException) {
            Exception nested = ((SAXException)e).getException();
            if (nested != null) {
                e = nested;
            }
        }
        e.printStackTrace(System.err);
    }
    return document;
 }
 //
 // MAIN
 //

    /** Main program entry point. */
    public static void main(String argv[])
    throws Exception 
    {
        // is there anything to do?
        if (argv.length < 1) {
            System.exit(1);
        }
        String fileName = argv[0];
        
        System.out.println("\n  <> File to process: " + fileName);
       
       // Hashtable registrationsHash = XMLToCSITranslator.translateRegistrations(fileName);
       // System.out.println("\n  <> Translated registrations: " + registrationsHash);
        
        Hashtable commands = XMLToCSITranslator.translateActionsFromXMLFile(fileName);
        System.out.println("\n  <> Translated commands: " + commands);
       
        synchronized ( Thread.currentThread() ) {
            Thread.currentThread().wait();
        }

    } // main(String[])

    public static String getXMLFileAsString(String xmlFileName)
	{
        try{
            InputSource inputSource = new InputSource(xmlFileName);
            Document document = _builder.parse(inputSource);
            StringWriter sw = new StringWriter();
            Transformer serializer = TransformerFactory.newInstance().newTransformer();
            serializer.transform(new DOMSource(document), new StreamResult(sw));
            return sw.toString();
        }
        catch (Exception e) {
            e.printStackTrace();
            return null;
        }
  
	}
  
} // class XMLToCSITranslator
