/**
 * Last Modified by: $Author: auszok $
 * On: $Date: 2005/12/09 13:21:27 $
 */
 
package kaos.xmlschema.classifier;

import kaos.core.csi.extension.KAoSExtensionComponentImpl;
import kaos.core.csi.extension.classification.InstanceClassifier;
import kaos.policy.information.OntologyInterfaces;

import kaos.ontology.management.UnknownConceptException;

import kaos.xmlschema.util.XMLSchemaBuilderImpl;

import javax.agent.service.directory.DirectoryFailure;

import java.util.Hashtable;
import java.util.Vector;

/**   
 *   Implementaion of the XMLSchema value classifier
 *   
 *   $Revision: 1.10 $
 */
public class XMLSchemaClassifier extends KAoSExtensionComponentImpl implements InstanceClassifier 
{      
	private Hashtable _cachedSchemas;
	private OntologyInterfaces _dirProxy;
        
	public XMLSchemaClassifier(OntologyInterfaces proxy)
	{
        super("XMLSchemaClassifier", "");
        
	    setOntologyInterfaces(proxy);
		_cachedSchemas = new Hashtable();
	}
	
	public XMLSchemaClassifier()
	{
        super("XMLSchemaClassifier", "");
        
	    _cachedSchemas = new Hashtable();
	}
	
    /**
     * Check if the instance is in the given class range 
     *
     * @param className                 ontology name of the class defining the range 
     *
     * @param instance				    the instance to be checked
     * 
     * @param classDesc					not used (should be passed null)
     * 
     * @param instDesc					not used (should be passed null)	
     *
     * @return                          True if the instance is of the specified class, false if not.
     *
     * @exception                       throws UnknownConceptException if it is not possoble to obtain information about the schema
     */
    public boolean classify(String className, Object classDesc, String instance, Object instDesc) throws UnknownConceptException
	{	
        // Get the schema description for the specified name of the schema
		Vector schema = (Vector)_cachedSchemas.get(className);
		
        // If schema is not cached and the access to outside repository is activated
		if ((schema == null) && (_dirProxy != null))
			// must make a call to directory service to get the schema for the given data type...	
			try
			{
				schema = _dirProxy.getSchema(className);
			}
			catch(DirectoryFailure e)
			{
				throw new UnknownConceptException("unable to contact directory to get XML schema for " + className);
			}
		
        // Check the schema variable again if it was initilized using the outside repository
		if (schema == null)
			// the directory does not contain the schema
			throw new UnknownConceptException("XMLSchema not found for " + className);
        
		XMLSchemaBuilderImpl xsb = null;
		
		try
		{
            // parse the XML Schema
			xsb = new XMLSchemaBuilderImpl(schema);						
			_cachedSchemas.put(xsb.getCurrentDataTypeName(), xsb);
		}	
		catch(Exception e)
		{
			e.printStackTrace();
		}
		
        // If sucessful
		if (xsb != null)
            // Use the library to perfom the classification; by asking if the instance is a valid instance of teh given schema
			return xsb.isValid((String)instance);	
        else 
            return false;
	}
	
    /**
     * Set OntologyInterfaces.
     *
     * @param ontInterfaces   OntologyInterfaces to be used, if needed for instance classification.
     */
    public void setOntologyInterfaces(OntologyInterfaces ontInterfaces)
    {
        _dirProxy = ontInterfaces;
    }
}
